# This function implements ASF, QSF and DSF estimation as described in the paper.
# Called by 'EA_mfile.R'.
# Should be used for inference - else use strucfuncs() for estimation.

# This version: June 2019.


#====================================================================
#
#				                     ESTIMATION        
#            
#====================================================================


if(length(Qreg)>0){
resQR     <- strucfuncs(Y = Y, X = X, Z = Z, R = R, integration = integration, tausqsf = tausqsf, taus1 = taus1, taus2 = taus2,
                        trim = trim, qY = qY, qYbar = qYbar, gridX = gridX, FS = "QR", SS = "QR", rSS = rSS);

qsffitQR  <- resQR$QSF
asffitQR  <- resQR$ASF
dsffitQR  <- resQR$DSF
}
if(length(Dreg)>0){
resDR     <- strucfuncs(Y = Y, X = X, Z = Z, R = R, integration = integration, tausqsf = tausqsf, taus1 = taus1, taus2 = taus2,
                        trim = trim, qY = qY, qYbar = qYbar, gridX = gridX, FS = "DR", SS = "DR", rSS = rSS);
qsffitDR  <- resDR$QSF
asffitDR  <- resDR$ASF
dsffitDR  <- resDR$DSF
}

nobs  	   <- length(Y);


# ===================================================================
#                                           
#  			                     INFERENCE                   
#                                           
# ===================================================================


if(length(se) > 0){
  
if(length(qYbar)>0){ ngridY <- length(qYbar) }
  
#--------------------------------------------------------------------
# Storage
#--------------------------------------------------------------------
  
qsf.bootsQR    <- array(0, c(ngridX, B, ntausqsf))
qsf.bootsDR    <- array(0, c(ngridX, B, ntausqsf))
asf.bootsQR    <- matrix(0, ngridX, B)
asf.bootsDR    <- matrix(0, ngridX, B)
dsf.bootsQR    <- array(0, c(ngridY, B, ngridX))
dsf.bootsDR    <- array(0, c(ngridY, B, ngridX))


#--------------------------------------------------------------------
# Weighted bootstrap
#--------------------------------------------------------------------

if(length(Qreg)>0){
# QR s.e.
QRse <- foreach(s = 1:B, .packages = c("quantreg"), .options.RNG = seed) %dorng% { 
                strucfuncs(Y = Y, X = X, Z = Z, R = R, integration = integration, tausqsf = tausqsf, taus1 = taus1, taus2 = taus2,  
                           trim = trim, qY = qY, qYbar = qYbar, gridX = gridX, FS = "QR", SS = "QR", weights = wboots[,s])
}  

for(s in 1:B){
      qsf.bootsQR[,s,]   <- QRse[[s]]$QSF
      asf.bootsQR[,s]    <- QRse[[s]]$ASF     
      dsf.bootsQR[,s,]   <- QRse[[s]]$DSF
      }


print("QRse done !")
}

if(length(Dreg)>0){
# DR s.e. 
DRse <- foreach(s = 1:B, .packages = c("speedglm"), .options.RNG = seed) %dorng% { 
#DRse <- foreach(s = 1:B, .options.RNG = seed) %dorng% { 
                strucfuncs(Y = Y, X = X, Z = Z, R = R, integration = integration, tausqsf = tausqsf, taus1 = taus1, taus2 = taus2,  
                           trim = trim, qY = qY, qYbar = qYbar, gridX = gridX, FS = "DR", SS = "DR", link = link, rSS = rSS, weights = wboots[,s])
                }                          

for(s in 1:B){
      qsf.bootsDR[,s,] <- DRse[[s]]$QSF
      asf.bootsDR[,s]  <- DRse[[s]]$ASF     
      dsf.bootsDR[,s,] <- DRse[[s]]$DSF   
      }

print("DRse done !")
}

#--------------------------------------------------------------------
# Store results
#--------------------------------------------------------------------

sd.qsfQR          <- matrix(0, ngridX, ntausqsf)         # This the variance across bootstrap estimates, for fixed X, fixed U.
sd.qsfDR          <- matrix(0, ngridX, ntausqsf)         # This the variance across bootstrap estimates, for fixed X, fixed U.
sd.asfQR          <- matrix(0, ngridX, 1)                            # This the variance across bootstrap estimates, for fixed X, fixed U.
sd.asfDR          <- matrix(0, ngridX, 1)                            # This the variance across bootstrap estimates, for fixed X, fixed U.
sd.dsfQR          <- matrix(0, ngridY, ngridX)         # This the variance across bootstrap estimates, for fixed X, fixed U.
sd.dsfDR          <- matrix(0, ngridY, ngridX)         # This the variance across bootstrap estimates, for fixed X, fixed U.
K.qsfQR           <- array(0, c(ngridX, B, ntausqsf))               # get tnstarb's
K.qsfDR           <- array(0, c(ngridX, B, ntausqsf))               # get tnstarb's
K.asfQR           <- matrix(0, c(ngridX, B))               # get tnstarb's
K.asfDR           <- matrix(0, c(ngridX, B))               # get tnstarb's
K.dsfQR           <- array(0, c(ngridY, B, ngridX))               # get tnstarb's
K.dsfDR           <- array(0, c(ngridY, B, ngridX))               # get tnstarb's
qsf.se.wQR        <- matrix(0, ngridX, ntausqsf)
qsf.se.wDR        <- matrix(0, ngridX, ntausqsf)
qsf.lbwQR         <- matrix(0, ngridX, ntausqsf)
qsf.lbwDR         <- matrix(0, ngridX, ntausqsf)
qsf.ubwQR         <- matrix(0, ngridX, ntausqsf)
qsf.ubwDR         <- matrix(0, ngridX, ntausqsf)
dsf.se.wQR        <- matrix(0, ngridY, ngridX)
dsf.se.wDR        <- matrix(0, ngridY, ngridX)
dsf.lbwQR         <- matrix(0, ngridY, ngridX)
dsf.lbwDR         <- matrix(0, ngridY, ngridX)
dsf.ubwQR         <- matrix(0, ngridY, ngridX)
dsf.ubwDR         <- matrix(0, ngridY, ngridX)
tnstarbQR         <- array(0, c(ngridX, ntausqsf, B))
tnstarbDR         <- array(0, c(ngridX, ntausqsf, B))
tnstarbdQR        <- array(0, c(ngridY, ngridX, B))
tnstarbdDR        <- array(0, c(ngridY, ngridX, B))

# s.d.
  for(i in 1:ntausqsf){
    if(length(Qreg)>0){ sd.qsfQR[,i] <- apply(qsf.bootsQR[,,i], 1, IQR, na.rm = T)/1.349 }
    if(length(Dreg)>0){ sd.qsfDR[,i] <- apply(qsf.bootsDR[,,i], 1, IQR, na.rm = T)/1.349 }
  }

  for(i in 1:ngridX){
    if(length(Qreg)>0){ sd.dsfQR[,i] <- apply(dsf.bootsQR[,,i], 1, IQR, na.rm = T)/1.349 }
    if(length(Dreg)>0){ sd.dsfDR[,i] <- apply(dsf.bootsDR[,,i], 1, IQR, na.rm = T)/1.349 }
  }

if(length(Qreg)>0){ sd.asfQR <- apply(asf.bootsQR, 1, IQR, na.rm = T)/1.349 }
if(length(Dreg)>0){ sd.asfDR <- apply(asf.bootsDR, 1, IQR, na.rm = T)/1.349 }

# t-stat
  for(i in 1:ntausqsf){
    if(length(Qreg)>0){ K.qsfQR[,,i] <- (qsf.bootsQR[,,i] - matrix(qsffitQR[,i], ngridX, B, byrow = F)) / matrix(sd.qsfQR[,i], ngridX, B, byrow = F) }
    if(length(Dreg)>0){ K.qsfDR[,,i] <- (qsf.bootsDR[,,i] - matrix(qsffitDR[,i], ngridX, B, byrow = F)) / matrix(sd.qsfDR[,i], ngridX, B, byrow = F) }
        }
  for(i in 1:ngridX){
    if(length(Qreg)>0){ K.dsfQR[,,i] <- (dsf.bootsQR[,,i] - matrix(dsffitQR[,i], ngridY, B, byrow = F)) / matrix(sd.dsfQR[,i], ngridY, B, byrow = F) }
    if(length(Dreg)>0){ K.dsfDR[,,i] <- (dsf.bootsDR[,,i] - matrix(dsffitDR[,i], ngridY, B, byrow = F)) / matrix(sd.dsfDR[,i], ngridY, B, byrow = F) }
  }

  if(length(Qreg)>0){ K.asfQR <- (asf.bootsQR - matrix(asffitQR, ngridX, B, byrow = F)) / matrix(sd.asfQR, ngridX, B, byrow = F) }
  if(length(Dreg)>0){ K.asfDR <- (asf.bootsDR - matrix(asffitDR, ngridX, B, byrow = F)) / matrix(sd.asfDR, ngridX, B, byrow = F) }
  for(s in 1:B){
    if(length(Qreg)>0){
        iInf  <- which(K.qsfQR[,s,] == Inf)
        imInf <- which(K.qsfQR[,s,] == -Inf)
        if(length(c(iInf,imInf)>0)){ K.qsfQR[,s,][c(iInf,imInf)] <- NaN }
        tnstarbQR[,,s]   <- K.qsfQR[,s,]                                            # This is just some rearranging
        tnstarbdQR[,,s]  <- K.dsfQR[,s,]                                            # This is just some rearranging
        }
    if(length(Dreg)>0){
        iInf  <- which(K.qsfDR[,s,] == Inf)
        imInf <- which(K.qsfDR[,s,] == -Inf)
        if(length(c(iInf,imInf)>0)){ K.qsfDR[,s,][c(iInf,imInf)] <- NaN }
        tnstarbDR[,,s]   <- K.qsfDR[,s,]                                            # To be fixed
        tnstarbdDR[,,s]  <- K.dsfDR[,s,]                                            # To be fixed
    }
  }
  if(length(Qreg)>0){ 
    tnstaraQR         <- K.asfQR 
    Kmax.asfQR 	      <- apply(tnstaraQR, 2, FUN = max, na.rm = T);
    Kmax.qsfQR 	      <- apply(tnstarbQR, 3, FUN = max, na.rm = T);
    Kmax.dsfQR 	      <- apply(tnstarbdQR, 3, FUN = max, na.rm = T);
    Kalpha.asfQR      <- quantile(Kmax.asfQR, 1 - alphab);
    Kalpha.qsfQR      <- quantile(Kmax.qsfQR, 1 - alphab);
    Kalpha.dsfQR      <- quantile(Kmax.dsfQR, 1 - alphab);
    }
  if(length(Dreg)>0){ 
    tnstaraDR         <- K.asfDR
    Kmax.asfDR 	      <- apply(tnstaraDR, 2, FUN = max, na.rm = T);
    Kmax.qsfDR 	      <- apply(tnstarbDR, 3, FUN = max, na.rm = T);
    Kmax.dsfDR 	      <- apply(tnstarbdDR, 3, FUN = max, na.rm = T);
    Kalpha.asfDR      <- quantile(Kmax.asfDR, 1 - alphab);
    Kalpha.qsfDR      <- quantile(Kmax.qsfDR, 1 - alphab);
    Kalpha.dsfDR      <- quantile(Kmax.dsfDR, 1 - alphab);
    }


  for(i in 1:ntausqsf){
    if(length(Qreg)>0){
        qsf.lbwQR[,i] <- qsffitQR[,i] - Kalpha.qsfQR * sd.qsfQR[,i];
        qsf.ubwQR[,i] <- qsffitQR[,i] + Kalpha.qsfQR * sd.qsfQR[,i];
        }
    if(length(Dreg)>0){ 
        qsf.lbwDR[,i] <- qsffitDR[,i] - Kalpha.qsfDR * sd.qsfDR[,i];
        qsf.ubwDR[,i] <- qsffitDR[,i] + Kalpha.qsfDR * sd.qsfDR[,i];
    }
  }

  for(i in 1:ngridX){
    if(length(Qreg)>0){
        dsf.lbwQR[,i] <- dsffitQR[,i] - Kalpha.dsfQR * sd.dsfQR[,i];
        dsf.ubwQR[,i] <- dsffitQR[,i] + Kalpha.dsfQR * sd.dsfQR[,i];
        iloutQR <- which(dsf.lbwQR[,i]<0)
        iuoutQR <- which(dsf.ubwQR[,i]>1)
        if(length(iloutQR)>0){ dsf.lbwQR[iloutQR,i] <- 0 }
        if(length(iuoutQR)>0){ dsf.ubwQR[iuoutQR,i] <- 1 }
        }
    if(length(Dreg)>0){ 
        dsf.lbwDR[,i] <- dsffitDR[,i] - Kalpha.dsfDR * sd.dsfDR[,i];
        dsf.ubwDR[,i] <- dsffitDR[,i] + Kalpha.dsfDR * sd.dsfDR[,i];
        iloutDR <- which(dsf.lbwDR[,i]<0)
        iuoutDR <- which(dsf.ubwDR[,i]>1)
        if(length(iloutDR)>0){ dsf.lbwDR[iloutDR,i] <- 0 }
        if(length(iuoutDR)>0){ dsf.ubwDR[iuoutDR,i] <- 1 }
    }
  }
  
  if(length(Qreg)>0){
  asf.lbwQR         <- asffitQR - Kalpha.asfQR * sd.asfQR;
  asf.ubwQR         <- asffitQR + Kalpha.asfQR * sd.asfQR;
  }
  if(length(Dreg)>0){ 
  asf.lbwDR         <- asffitDR - Kalpha.asfDR * sd.asfDR;
  asf.ubwDR         <- asffitDR + Kalpha.asfDR * sd.asfDR;
  }

}


#====================================================================
# Sensitivity
#====================================================================

if(length(sensitivity) > 0 && integration == "Smplave"){
  
#--------------------------------------------------------------------
# V polys
#--------------------------------------------------------------------
print("Vpolys")
   
if(length(Qreg)>0){
resQR32   <- strucfuncs(Y = Y, X = X, Z = Z, R = R.sens, integration = integration, tausqsf = tausqsf, taus1 = taus1, taus2 = taus2,
                        trim = trim, qY = qY, gridX = gridX, FS = "QR", SS = "QR", Vpolys = 2);

resQR33   <- strucfuncs(Y = Y, X = X, Z = Z, R = R.sens, integration = integration, tausqsf = tausqsf, taus1 = taus1, taus2 = taus2,
                        trim = trim, qY = qY, gridX = gridX, FS = "QR", SS = "QR", Vpolys = 3);
}
if(length(Dreg)>0){
resDR32   <- strucfuncs(Y = Y, X = X, Z = Z, R = R.sens, integration = integration, tausqsf = tausqsf, taus1 = taus1, taus2 = taus2,
                        trim = trim, qY = qY, gridX = gridX, FS = "DR", SS = "DR", Vpolys = 2, rSS = rSS);

resDR33   <- strucfuncs(Y = Y, X = X, Z = Z, R = R.sens, integration = integration, tausqsf = tausqsf, taus1 = taus1, taus2 = taus2,
                        trim = trim, qY = qY, gridX = gridX, FS = "DR", SS = "DR", Vpolys = 3, rSS = rSS);
}

if(length(Qreg)>0){
#--------------------------------------------------------------------
# Cross validation
#--------------------------------------------------------------------
print("CV")
cv.val.QR    <- NULL
polyCV       <- 5
for(j in 1:polyCV){
    cv.val.QR <- c(cv.val.QR, crossvalidate(Y = Y, X = X, Z = Z, R = R.sens, Vpolys = j, taus = seq(.01,.99,length.out=599)) )
    }

#pdf( paste(good, "_CV_",sample,"_",link,"_nx",ngridX,".pdf", sep="") )
pdf( paste("Figure7_",good,"_SM.pdf", sep="") )
plot(seq(1,polyCV,by=1), round(cv.val.QR*1000,digits=3), xlab = "Polynomial degree", ylab = "Squared error", type = "l")
points(seq(1,polyCV,by=1)[which.min(round(cv.val.QR*1000,digits=3))], round(cv.val.QR*1000,digits=3)[which.min(round(cv.val.QR*1000,digits=3))], col = 2)
dev.off();

#--------------------------------------------------------------------
# Nonlinear specification
#--------------------------------------------------------------------
print("nonlin")
# QSF
tausqsf2    <- seq(.15, .85, by = .05);	# Quantiles of interest for the QSF
gridX       <- quantile(X, seq(.1, .9, length.out = 15))

# Splines
nknots      <- 4;
degree      <- 3;
knots_X	    <- quantile(X, c(1:nknots)/(nknots+1));
XSS         <- bs(X, degree = degree, intercept = FALSE, knots = knots_X);
ncsplines_X <- ncol(XSS)
splinefuns <- list()
for(l in 1:ncsplines_X){
  splinefuns[[l]] <- approxfun(X,XSS[,l])
}

res.nonlinQR   <- strucfuncs(Y = Y, X = X, XSS = XSS, Z = Z, R = R.sens, integration = integration, tausqsf = tausqsf2, taus1 = taus1, taus2 = taus2,
                             trim = trim, qY = qY, gridX = gridX, FS = "QR", SS = "QR", rSS = rSS, splinefuns = splinefuns);

res.nonlinDR   <- strucfuncs(Y = Y, X = X, XSS = XSS, Z = Z, R = R.sens, integration = integration, tausqsf = tausqsf2, taus1 = taus1, taus2 = taus2,
                        trim = trim, qY = qY, gridX = gridX, FS = "DR", SS = "DR", rSS = rSS, splinefuns = splinefuns);
}

}

#load("EstimationResultspm_Leisure_pool_Smplave_logit_ngFS599_ngSS599_ntqsf3_nx5t0_qY0.RData")
#results$nonlinDR  <- res.nonlinDR
#results$nonlinQR  <- res.nonlinQR

#===============================================================================
# Save results
#===============================================================================

  results           <- list()
  if(length(Qreg)>0){ results$QR <- resQR }
  if(length(Dreg)>0){ results$DR <- resDR }
  results$tausqsf   <- tausqsf

  if(length(se) > 0){
    if(length(Qreg)>0){
    results$qsf.lbwQR  <- qsf.lbwQR
    results$qsf.ubwQR  <- qsf.ubwQR
    results$asf.lbwQR  <- asf.lbwQR
    results$asf.ubwQR  <- asf.ubwQR
    results$dsf.lbwQR  <- dsf.lbwQR
    results$dsf.ubwQR  <- dsf.ubwQR
    }
    if(length(Dreg)>0){ 
    results$qsf.lbwDR  <- qsf.lbwDR
    results$qsf.ubwDR  <- qsf.ubwDR
    results$asf.lbwDR  <- asf.lbwDR
    results$asf.ubwDR  <- asf.ubwDR
    results$dsf.lbwDR  <- dsf.lbwDR
    results$dsf.ubwDR  <- dsf.ubwDR
    }
  }
  
  if(length(sensitivity) > 0){
    if(length(Dreg)>0){
      results$DRquad    <- resDR32
      results$DRcub     <- resDR33     
    }
    if(length(Qreg)>0){
      results$QRquad    <- resQR32
      results$QRcub     <- resQR33
      results$nonlinQR  <- res.nonlinQR
      results$nonlinDR  <- res.nonlinDR
    }
  }
  

if(length(smallgX)==0){
if(length(tmp)==0){
  if(length(R) == 0){ save(results, file = paste("EstimationResults_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",100*trim[1],"_qY",100*qY[1],".RData",sep="") ) }
  if(length(R) > 0){  save(results, file = paste("EstimationResultspm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",100*trim[1],"_qY",100*qY[1],".RData",sep="") ) }
}
if(length(tmp)>0){
save(results, file = paste("EstimationResultstmp_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",100*trim[1],"_qY",qY[1],".RData",sep="") ) 
}
}
  
if(length(smallgX)>0){
  if(length(R) == 0){ save(results, file = paste("EstimationResultssg_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",100*trim[1],"_qY",100*qY[1],".RData",sep="") ) }
  if(length(R) > 0){  save(results, file = paste("EstimationResultspmsg_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",100*trim[1],"_qY",100*qY[1],".RData",sep="") ) }
  }
rm(ls="results")


