# This is the file called by 'MC_mfile.R' where all parameter values are set.
#
#
# This version: June 2019.


#===============================================================================
#                                              
#			          	CALIBRATION                            
#                                              
#===============================================================================

# Version
#--------------------------------------------------------------------

#===============================================================================
# Data
#===============================================================================

if(sample == "pool"){   engelpool  <- read.dta("engelpool.dta") 
                        attach(engelpool) }

if(good == "Leisure") { Y <- leisure }
X  	          <- logexp;
Z             <- logwages;



# ==================================================
#                                     
#                  Calibration               
#                                    
# ==================================================
#cal.loc     <- califunc(Y = Y, X = X, Z = Z, qY = qY, taus1 = taus1, taus2 = taus2, trim = trim, dgp = "loc");
#cal.qr      <- califunc(Y = Y, X = X, Z = Z, qY = qY, taus1 = taus1, taus2 = taus2, trim = trim, dgp = "qr");
#cal.dr      <- califunc(Y = Y, X = X, Z = Z, qY = qY, taus1 = taus1, taus2 = taus2, trim = trim, dgp = "dr");
#cal <- list()
#cal[[1]] <- cal.loc
#cal[[2]] <- cal.qr
#cal[[3]] <- cal.dr
#save(cal, file = paste("CalibrationResults_",good,"ng",length(taus2),".RData", sep="") )

if(good == "Leisure"){ load("CalibrationResults_Leisureng599.Rdata") }

cal.loc <- cal[[1]]
cal.qr  <- cal[[2]]
cal.dr  <- cal[[3]]

a.loc       <- matrix(round(cal.loc$avec,digit=2), 1, 4)
a.qr        <- matrix(round(cal.qr$avec,digit=2), 1, 4)
a.dr        <- matrix(round(cal.dr$avec,digit=2), 1, 4)
b.loc       <- matrix(round(cal.loc$bvec,digit=2), 1, 4)
b.qr        <- matrix(round(cal.qr$bvec,digit=2), 1, 4)
b.dr        <- matrix(round(cal.dr$bvec,digit=2), 1, 4)

#b.dr[3] <- b.dr[3]*1.9
#b.dr[4] <- b.dr[4]*3.5
#b.dr[3] <- b.dr[3]*1.5
#b.dr[4] <- b.dr[4]*2
b.dr[3] <- b.dr[3]*1.5
b.dr[4] <- b.dr[4]*2.25



# ==================================================
#                                    
#          Simulations                
#                                    
# ==================================================

seed               <- 999
#Xgridsim           <- seq(quantile(X,.1), quantile(X,.9), length.out = ngridX)
Xgridsim           <- quantile(X, seq(.1, .9, length.out = ngridX))

qsf.sim.QR.loc     <- array(0, c(ngridX, nrep, ntausqsf))  
qsf.sim.DR.loc     <- array(0, c(ngridX, nrep, ntausqsf))  
asf.sim.QR.loc     <- matrix(0, ngridX, nrep)  
asf.sim.DR.loc     <- matrix(0, ngridX, nrep)  
cqf.sim.loc        <- array(0, c(ngridX, nrep, ntausqsf))
cmf.sim.loc        <- matrix(0, ngridX, nrep)

qsf.sim.QR.qr      <- array(0, c(ngridX, nrep, ntausqsf))  
qsf.sim.DR.qr      <- array(0, c(ngridX, nrep, ntausqsf))  
asf.sim.QR.qr      <- matrix(0, ngridX, nrep)  
asf.sim.DR.qr      <- matrix(0, ngridX, nrep)  
cqf.sim.qr         <- array(0, c(ngridX, nrep, ntausqsf))
cmf.sim.qr         <- matrix(0, ngridX, nrep)

qsf.sim.QR.dr      <- array(0, c(ngridX, nrep, ntausqsf))  
qsf.sim.DR.dr      <- array(0, c(ngridX, nrep, ntausqsf))  
asf.sim.QR.dr      <- matrix(0, ngridX, nrep)  
asf.sim.DR.dr      <- matrix(0, ngridX, nrep)  
cqf.sim.dr         <- array(0, c(ngridX, nrep, ntausqsf))
cmf.sim.dr         <- matrix(0, ngridX, nrep)

Y.loc              <- matrix(0, nsim, nrep)
X.loc              <- matrix(0, nsim, nrep)
Y.qr               <- matrix(0, nsim, nrep)
X.qr               <- matrix(0, nsim, nrep)
Y.dr               <- matrix(0, nsim, nrep)
X.dr               <- matrix(0, nsim, nrep)

qVhatQR.loc        <- matrix(0, nsim, nrep)
qVhatDR.loc        <- matrix(0, nsim, nrep)

qVhatQR.qr         <- matrix(0, nsim, nrep)
qVhatDR.qr         <- matrix(0, nsim, nrep)

qVhatQR.dr         <- matrix(0, nsim, nrep)
qVhatDR.dr         <- matrix(0, nsim, nrep)

coefsqr.loc        <- array(0, c(ntausqsf,2,nrep));
coefsqr.qr         <- array(0, c(ntausqsf,2,nrep));
coefsqr.dr         <- array(0, c(ntausqsf,2,nrep));
coefsls.loc        <- matrix(0,2,nrep);
coefsls.qr         <- matrix(0,2,nrep);
coefsls.dr         <- matrix(0,2,nrep);

set.seed(seed)

Zsim    <- Z
if(nsim != length(Y)){ Zsim  <- Z[sample(length(Y),nsim,replace=T) ] }
#Z2      <- Zsim
#Zsim    <- matrix(pnorm((Zsim-mean(Zsim))/sd(Zsim)),nsim,1)

# True A&QSF  
qsftrue.loc  <- matrix(0, ngridX, ntausqsf);
qsftrue.qr   <- matrix(0, ngridX, ntausqsf);
qsftrue.dr   <- matrix(0, ngridX, ntausqsf);
for(l in 1:ntausqsf){;
      qsftrue.loc[,l] <- b.loc[1:2] %*% rbind(1,Xgridsim) + (b.loc[3:4] %*% rbind(1,Xgridsim))*qnorm(tausqsf[l], sd = cal.loc$sde);
      qsftrue.qr[,l]  <- b.qr[1:2] %*% rbind(1,Xgridsim) + (b.qr[3:4] %*% rbind(1,Xgridsim))*qnorm(tausqsf[l]);
      qsftrue.dr[,l]  <- -(b.dr[1:2] %*% rbind(1,Xgridsim))/(b.dr[3:4] %*% rbind(1,Xgridsim)) + (1/(b.dr[3:4] %*% rbind(1,Xgridsim)))*qnorm(tausqsf[l]);
      };              
asftrue.loc <- as.numeric(b.loc[1:2] %*% rbind(1,Xgridsim));
asftrue.qr  <- as.numeric(b.qr[1:2] %*% rbind(1,Xgridsim));
asftrue.dr  <- as.numeric(-(b.dr[1:2] %*% rbind(1,Xgridsim))/(b.dr[3:4] %*% rbind(1,Xgridsim)));



for(i in 1:nrep){

# Generate a sample
    
    datsim.loc                <- simdata(avec = a.loc, bvec = b.loc, rho = rho, sde = cal.loc$sde, sdv = cal.loc$sdv, Z = Zsim, n = nsim, seed = seed, dgp = "loc", gridY = gridY);
    datsim.qr                 <- simdata(avec = a.qr, bvec = b.qr, rho = rho, Z = Zsim, n = nsim, seed = seed, dgp = "qr", gridY = gridY);
    datsim.dr                 <- simdata(avec = a.dr, bvec = b.dr, rho = rho, Z = Zsim, n = nsim, seed = seed, dgp = "dr", gridY = gridY);

    Y.loc[,i]                 <- matrix(datsim.loc$Y, nsim,1)
    X.loc[,i]                 <- matrix(datsim.loc$X, nsim,1)
    Y.qr[,i]                  <- matrix(datsim.qr$Y, nsim,1)
    X.qr[,i]                  <- matrix(datsim.qr$X, nsim,1)
    Y.dr[,i]                  <- matrix(datsim.dr$Y, nsim,1)
    X.dr[,i]                  <- matrix(datsim.dr$X, nsim,1)

# First Stage
    VhatDR.loc                <- CF(X = X.loc[,i], Z = Zsim, taus = taus1, link = link, FS = "DR")$CV;        	
    VhatQR.loc                <- CF(X = X.loc[,i], Z = Zsim, taus = taus1, FS = "QR")$CV;        		
    qVhatQR.loc[,i]           <- qnorm(VhatQR.loc)
    qVhatDR.loc[,i]           <- qnorm(VhatDR.loc)

    VhatDR.qr                 <- CF(X = X.qr[,i], Z = Zsim, taus = taus1, link = link, FS = "DR")$CV;        		    
    VhatQR.qr   	            <- CF(X = X.qr[,i], Z = Zsim, taus = taus1, FS = "QR")$CV;        		
    qVhatQR.qr[,i]            <- qnorm(VhatQR.qr)
    qVhatDR.qr[,i]            <- qnorm(VhatDR.qr)

    VhatDR.dr                 <- CF(X = X.dr[,i], Z = Zsim, taus = taus1, link = link, FS = "DR")$CV;        		    
    VhatQR.dr   	            <- CF(X = X.dr[,i], Z = Zsim, taus = taus1, FS = "QR")$CV;        		
    qVhatQR.dr[,i]            <- qnorm(VhatQR.dr)
    qVhatDR.dr[,i]            <- qnorm(VhatDR.dr)
    
    fitls.loc                 <- lm(Y.loc[,i] ~ X.loc[,i])
    fitls.qr                  <- lm(Y.qr[,i] ~ X.qr[,i])
    fitls.dr                  <- lm(Y.dr[,i] ~ X.dr[,i])
    
    coefsls.loc[,i]           <- t(coef(fitls.loc));
    coefsls.qr[,i]            <- t(coef(fitls.qr));
    coefsls.dr[,i]            <- t(coef(fitls.dr));
    
    fitqr.loc                 <- rq(Y.loc[,i] ~ X.loc[,i], tau = tausqsf);    
    fitqr.qr                  <- rq(Y.qr[,i] ~ X.qr[,i], tau = tausqsf);    
    fitqr.dr                  <- rq(Y.dr[,i] ~ X.dr[,i], tau = tausqsf);    
    
    coefsqr.loc[,,i]          <- t(coef(fitqr.loc));
    coefsqr.qr[,,i]           <- t(coef(fitqr.qr));
    coefsqr.dr[,,i]           <- t(coef(fitqr.dr));
    
    }

# Second and Third Stages
    resQSF.QR.loc             <- foreach(i = 1:nrep, .packages = c("quantreg"), .options.RNG = seed) %dorng% { 
                                         strucfuncs(Y = Y.loc[,i], X = X.loc[,i], qVhat = qVhatQR.loc[,i], tausqsf = tausqsf, taus2 = taus2, trim = trim, qY = qY, gridX = Xgridsim, SS = "QR");
                                         }
    resQSF.DR.loc             <- foreach(i = 1:nrep, .packages = c("speedglm"), .options.RNG = seed) %dorng% {  
                                         strucfuncs(Y = Y.loc[,i], X = X.loc[,i], qVhat = qVhatDR.loc[,i], tausqsf = tausqsf, taus2 = taus2, trim = trim, qY = qY, gridX = Xgridsim, SS = "DR");
                                         }
    
    resQSF.QR.qr              <- foreach(i = 1:nrep, .packages = c("quantreg"), .options.RNG = seed) %dorng% { 
                                         strucfuncs(Y = Y.qr[,i], X = X.qr[,i], qVhat = qVhatQR.qr[,i], tausqsf = tausqsf, taus2 = taus2, trim = trim, qY = qY, gridX = Xgridsim, SS = "QR");
                                         }
    resQSF.DR.qr              <- foreach(i = 1:nrep, .packages = c("speedglm"), .options.RNG = seed) %dorng% {  
                                         strucfuncs(Y = Y.qr[,i], X = X.qr[,i], qVhat = qVhatDR.qr[,i], tausqsf = tausqsf, taus2 = taus2, trim = trim, qY = qY, gridX = Xgridsim, SS = "DR");
                                         }
    
    resQSF.QR.dr              <- foreach(i = 1:nrep, .packages = c("quantreg"), .options.RNG = seed) %dorng% { 
                                         strucfuncs(Y = Y.dr[,i], X = X.dr[,i], qVhat = qVhatQR.dr[,i], tausqsf = tausqsf, taus2 = taus2, trim = trim, qY = qY, gridX = Xgridsim, SS = "QR");
                                         }
    resQSF.DR.dr              <- foreach(i = 1:nrep, .packages = c("speedglm"), .options.RNG = seed) %dorng% {  
                                         strucfuncs(Y = Y.dr[,i], X = X.dr[,i], qVhat = qVhatDR.dr[,i], tausqsf = tausqsf, taus2 = taus2, trim = trim, qY = qY, gridX = Xgridsim, SS = "DR");
                                         }

 
    
for(i in 1:nrep){
  
    qsf.sim.QR.loc[,i,]     <- resQSF.QR.loc[[i]]$QSF
    qsf.sim.DR.loc[,i,]     <- resQSF.DR.loc[[i]]$QSF
    cqf.sim.loc[,i,]        <- t(coefsqr.loc[,,i] %*% rbind(rep(1, ngridX), t(Xgridsim)))
    
    qsf.sim.QR.qr[,i,]      <- resQSF.QR.qr[[i]]$QSF
    qsf.sim.DR.qr[,i,]      <- resQSF.DR.qr[[i]]$QSF
    cqf.sim.qr[,i,]         <- t(coefsqr.qr[,,i] %*% rbind(rep(1, ngridX), t(Xgridsim)));
    
    qsf.sim.QR.dr[,i,]      <- resQSF.QR.dr[[i]]$QSF
    qsf.sim.DR.dr[,i,]      <- resQSF.DR.dr[[i]]$QSF
    cqf.sim.dr[,i,]         <- t(coefsqr.dr[,,i] %*% rbind(rep(1, ngridX), t(Xgridsim)));
    
    asf.sim.QR.loc[,i]      <- resQSF.QR.loc[[i]]$ASF
    asf.sim.DR.loc[,i]      <- resQSF.DR.loc[[i]]$ASF
    cmf.sim.loc[,i]         <- coefsls.loc[,i] %*% rbind(rep(1, ngridX), t(Xgridsim));
    
    asf.sim.QR.qr[,i]       <- resQSF.QR.qr[[i]]$ASF
    asf.sim.DR.qr[,i]       <- resQSF.DR.qr[[i]]$ASF
    cmf.sim.qr[,i]          <- coefsls.qr[,i] %*% rbind(rep(1, ngridX), t(Xgridsim));
    
    asf.sim.QR.dr[,i]       <- resQSF.QR.dr[[i]]$ASF
    asf.sim.DR.dr[,i]       <- resQSF.DR.dr[[i]]$ASF
    cmf.sim.dr[,i]          <- coefsls.dr[,i] %*% rbind(rep(1, ngridX), t(Xgridsim));
}


qsf.25.QR.loc   	<- matrix(0, ngridX, 3);
qsf.5.QR.loc  		<- matrix(0, ngridX, 3);
qsf.75.QR.loc 		<- matrix(0, ngridX, 3);
qsf.25.DR.loc  	  <- matrix(0, ngridX, 3);
qsf.5.DR.loc   	  <- matrix(0, ngridX, 3);
qsf.75.DR.loc  	  <- matrix(0, ngridX, 3);
RMSE.loc          <- array(0, c(4, 3, ntausqsf));
cqf.25.loc        <- matrix(0, ngridX, 3);
cqf.5.loc         <- matrix(0, ngridX, 3);
cqf.75.loc        <- matrix(0, ngridX, 3);

qsf.25.QR.qr 		  <- matrix(0, ngridX, 3);
qsf.5.QR.qr  		  <- matrix(0, ngridX, 3);
qsf.75.QR.qr 		  <- matrix(0, ngridX, 3);
qsf.25.DR.qr  	  <- matrix(0, ngridX, 3);
qsf.5.DR.qr   	  <- matrix(0, ngridX, 3);
qsf.75.DR.qr  	  <- matrix(0, ngridX, 3);
RMSE.qr           <- array(0, c(4, 3, ntausqsf));
cqf.25.qr         <- matrix(0, ngridX, 3);
cqf.5.qr          <- matrix(0, ngridX, 3);
cqf.75.qr         <- matrix(0, ngridX, 3);

qsf.25.QR.dr 		  <- matrix(0, ngridX, 3);
qsf.5.QR.dr  		  <- matrix(0, ngridX, 3);
qsf.75.QR.dr 		  <- matrix(0, ngridX, 3);
qsf.25.DR.dr  	  <- matrix(0, ngridX, 3);
qsf.5.DR.dr   	  <- matrix(0, ngridX, 3);
qsf.75.DR.dr  	  <- matrix(0, ngridX, 3);
RMSE.dr           <- array(0, c(4, 3, ntausqsf));
cqf.25.dr         <- matrix(0, ngridX, 3);
cqf.5.dr          <- matrix(0, ngridX, 3);
cqf.75.dr         <- matrix(0, ngridX, 3);

asf.QR.loc   	    <- matrix(0, ngridX, 3);
asf.DR.loc  	    <- matrix(0, ngridX, 3);
RMSE.ASF.loc      <- matrix(0, 4, 3);
cmf.loc           <- matrix(0, ngridX, 3);

asf.QR.qr   	    <- matrix(0, ngridX, 3);
asf.DR.qr  	      <- matrix(0, ngridX, 3);
RMSE.ASF.qr       <- matrix(0, 4, 3);
cmf.qr            <- matrix(0, ngridX, 3);

asf.QR.dr   	    <- matrix(0, ngridX, 3);
asf.DR.dr  	      <- matrix(0, ngridX, 3);
RMSE.ASF.dr       <- matrix(0, 4, 3);
cmf.dr            <- matrix(0, ngridX, 3);


for(i in 1:ngridX){;
  
  qsf.25.QR.loc[i,]    <- quantile(qsf.sim.QR.loc[i,,1], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.5.QR.loc[i,]     <- quantile(qsf.sim.QR.loc[i,,2], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.75.QR.loc[i,]    <- quantile(qsf.sim.QR.loc[i,,3], probs = c(.25, .5, .75), na.rm = TRUE);
  
  qsf.25.DR.loc[i,]    <- quantile(qsf.sim.DR.loc[i,,1], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.5.DR.loc[i,]     <- quantile(qsf.sim.DR.loc[i,,2], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.75.DR.loc[i,]    <- quantile(qsf.sim.DR.loc[i,,3], probs = c(.25, .5, .75), na.rm = TRUE);
  
  cqf.25.loc[i,]       <- quantile(cqf.sim.loc[i,,1], probs = c(.25, .5, .75), na.rm = TRUE);
  cqf.5.loc[i,]        <- quantile(cqf.sim.loc[i,,2], probs = c(.25, .5, .75), na.rm = TRUE);
  cqf.75.loc[i,]       <- quantile(cqf.sim.loc[i,,3], probs = c(.25, .5, .75), na.rm = TRUE);
  
  
  qsf.25.QR.qr[i,]     <- quantile(qsf.sim.QR.qr[i,,1], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.5.QR.qr[i,]      <- quantile(qsf.sim.QR.qr[i,,2], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.75.QR.qr[i,]     <- quantile(qsf.sim.QR.qr[i,,3], probs = c(.25, .5, .75), na.rm = TRUE);
  
  qsf.25.DR.qr[i,]     <- quantile(qsf.sim.DR.qr[i,,1], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.5.DR.qr[i,]      <- quantile(qsf.sim.DR.qr[i,,2], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.75.DR.qr[i,]     <- quantile(qsf.sim.DR.qr[i,,3], probs = c(.25, .5, .75), na.rm = TRUE);
  
  cqf.25.qr[i,]        <- quantile(cqf.sim.qr[i,,1], probs = c(.25, .5, .75), na.rm = TRUE);
  cqf.5.qr[i,]         <- quantile(cqf.sim.qr[i,,2], probs = c(.25, .5, .75), na.rm = TRUE);
  cqf.75.qr[i,]        <- quantile(cqf.sim.qr[i,,3], probs = c(.25, .5, .75), na.rm = TRUE);
  
  qsf.25.QR.dr[i,]     <- quantile(qsf.sim.QR.dr[i,,1], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.5.QR.dr[i,]      <- quantile(qsf.sim.QR.dr[i,,2], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.75.QR.dr[i,]     <- quantile(qsf.sim.QR.dr[i,,3], probs = c(.25, .5, .75), na.rm = TRUE);
  
  qsf.25.DR.dr[i,]     <- quantile(qsf.sim.DR.dr[i,,1], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.5.DR.dr[i,]      <- quantile(qsf.sim.DR.dr[i,,2], probs = c(.25, .5, .75), na.rm = TRUE);
  qsf.75.DR.dr[i,]     <- quantile(qsf.sim.DR.dr[i,,3], probs = c(.25, .5, .75), na.rm = TRUE);
  
  cqf.25.dr[i,]        <- quantile(cqf.sim.dr[i,,1], probs = c(.25, .5, .75), na.rm = TRUE);
  cqf.5.dr[i,]         <- quantile(cqf.sim.dr[i,,2], probs = c(.25, .5, .75), na.rm = TRUE);
  cqf.75.dr[i,]        <- quantile(cqf.sim.dr[i,,3], probs = c(.25, .5, .75), na.rm = TRUE);
  
  
  asf.QR.loc[i,]       <- quantile(asf.sim.QR.loc[i,], probs = c(.25, .5, .75), na.rm = TRUE);
  asf.DR.loc[i,]       <- quantile(asf.sim.DR.loc[i,], probs = c(.25, .5, .75), na.rm = TRUE);
  cmf.loc[i,]          <- quantile(cmf.sim.loc[i,], probs = c(.25, .5, .75), na.rm = TRUE);
  
  asf.QR.qr[i,]        <- quantile(asf.sim.QR.qr[i,], probs = c(.25, .5, .75), na.rm = TRUE);
  asf.DR.qr[i,]        <- quantile(asf.sim.DR.qr[i,], probs = c(.25, .5, .75), na.rm = TRUE);
  cmf.qr[i,]           <- quantile(cmf.sim.qr[i,], probs = c(.25, .5, .75), na.rm = TRUE);
  
  asf.QR.dr[i,]        <- quantile(asf.sim.QR.dr[i,], probs = c(.25, .5, .75), na.rm = TRUE);
  asf.DR.dr[i,]        <- quantile(asf.sim.DR.dr[i,], probs = c(.25, .5, .75), na.rm = TRUE);
  cmf.dr[i,]           <- quantile(cmf.sim.dr[i,], probs = c(.25, .5, .75), na.rm = TRUE);
  
};

# RMSE.loc
RMSE.loc[1,1,1]   <- mean(apply(abs(qsf.sim.QR.loc[,,1]-qsftrue.loc[,1]),1,mean))
RMSE.loc[2,1,1]   <- mean(apply(abs(qsf.sim.DR.loc[,,1]-qsftrue.loc[,1]),1,mean))             
RMSE.loc[3,1,1]   <- 100/1000 * (RMSE.loc[2,1,1]/RMSE.loc[1,1,1])
RMSE.loc[4,1,1]   <- mean(apply(abs(cqf.sim.loc[,,1]-qsftrue.loc[,1]),1,mean))  

RMSE.loc[1,1,2]   <- mean(apply(abs(qsf.sim.QR.loc[,,2]-qsftrue.loc[,2]),1,mean))
RMSE.loc[2,1,2]   <- mean(apply(abs(qsf.sim.DR.loc[,,2]-qsftrue.loc[,2]),1,mean))             
RMSE.loc[3,1,2]   <- 100/1000 * (RMSE.loc[2,1,2]/RMSE.loc[1,1,2])
RMSE.loc[4,1,2]   <- mean(apply(abs(cqf.sim.loc[,,2]-qsftrue.loc[,2]),1,mean))  

RMSE.loc[1,1,3]   <- mean(apply(abs(qsf.sim.QR.loc[,,3]-qsftrue.loc[,3]),1,mean))
RMSE.loc[2,1,3]   <- mean(apply(abs(qsf.sim.DR.loc[,,3]-qsftrue.loc[,3]),1,mean))             
RMSE.loc[3,1,3]   <- 100/1000 * (RMSE.loc[2,1,3]/RMSE.loc[1,1,3])
RMSE.loc[4,1,3]   <- mean(apply(abs(cqf.sim.loc[,,3]-qsftrue.loc[,3]),1,mean))  


RMSE.loc[1,2,1]   <- mean(sqrt(apply((qsf.sim.QR.loc[,,1]-qsftrue.loc[,1])^2, 1, mean)))
RMSE.loc[2,2,1]   <- mean(sqrt(apply((qsf.sim.DR.loc[,,1]-qsftrue.loc[,1])^2, 1, mean)))
RMSE.loc[3,2,1]   <- 100/1000 * (RMSE.loc[2,2,1]/RMSE.loc[1,2,1])
RMSE.loc[4,2,1]   <- mean(sqrt(apply((cqf.sim.loc[,,1]-qsftrue.loc[,1])^2, 1, mean)))  

RMSE.loc[1,2,2]   <- mean(sqrt(apply((qsf.sim.QR.loc[,,2]-qsftrue.loc[,2])^2, 1, mean)))
RMSE.loc[2,2,2]   <- mean(sqrt(apply((qsf.sim.DR.loc[,,2]-qsftrue.loc[,2])^2, 1, mean)))
RMSE.loc[3,2,2]   <- 100/1000 * (RMSE.loc[2,2,2]/RMSE.loc[1,2,2])
RMSE.loc[4,2,2]   <- mean(sqrt(apply((cqf.sim.loc[,,2]-qsftrue.loc[,2])^2, 1, mean)))  

RMSE.loc[1,2,3]   <- mean(sqrt(apply((qsf.sim.QR.loc[,,3]-qsftrue.loc[,3])^2, 1, mean)))
RMSE.loc[2,2,3]   <- mean(sqrt(apply((qsf.sim.DR.loc[,,3]-qsftrue.loc[,3])^2, 1, mean)))
RMSE.loc[3,2,3]   <- 100/1000 * (RMSE.loc[2,2,3]/RMSE.loc[1,2,3])
RMSE.loc[4,2,3]   <- mean(sqrt(apply((cqf.sim.loc[,,3]-qsftrue.loc[,3])^2, 1, mean)))  


RMSE.loc[1,3,1]   <- max(apply(abs(qsf.sim.QR.loc[,,1]-qsftrue.loc[,1]),1,mean))
RMSE.loc[2,3,1]   <- max(apply(abs(qsf.sim.DR.loc[,,1]-qsftrue.loc[,1]),1,mean))             
RMSE.loc[3,3,1]   <- 100/1000 * (RMSE.loc[2,3,1]/RMSE.loc[1,3,1])
RMSE.loc[4,3,1]   <- max(apply(abs(cqf.sim.loc[,,1]-qsftrue.loc[,1]),1,mean))  

RMSE.loc[1,3,2]   <- max(apply(abs(qsf.sim.QR.loc[,,2]-qsftrue.loc[,2]),1,mean))
RMSE.loc[2,3,2]   <- max(apply(abs(qsf.sim.DR.loc[,,2]-qsftrue.loc[,2]),1,mean))             
RMSE.loc[3,3,2]   <- 100/1000 * (RMSE.loc[2,3,2]/RMSE.loc[1,3,2])
RMSE.loc[4,3,2]   <- max(apply(abs(cqf.sim.loc[,,2]-qsftrue.loc[,2]),1,mean))  

RMSE.loc[1,3,3]   <- max(apply(abs(qsf.sim.QR.loc[,,3]-qsftrue.loc[,3]),1,mean))
RMSE.loc[2,3,3]   <- max(apply(abs(qsf.sim.DR.loc[,,3]-qsftrue.loc[,3]),1,mean))             
RMSE.loc[3,3,3]   <- 100/1000 * (RMSE.loc[2,3,3]/RMSE.loc[1,3,3])
RMSE.loc[4,3,3]   <- max(apply(abs(cqf.sim.loc[,,3]-qsftrue.loc[,3]),1,mean))  


# RMSE.qr
RMSE.qr[1,1,1]   <- mean(apply(abs(qsf.sim.QR.qr[,,1]-qsftrue.qr[,1]),1,mean))
RMSE.qr[2,1,1]   <- mean(apply(abs(qsf.sim.DR.qr[,,1]-qsftrue.qr[,1]),1,mean))             
RMSE.qr[3,1,1]   <- 100/1000 * (RMSE.qr[2,1,1]/RMSE.qr[1,1,1])
RMSE.qr[4,1,1]   <- mean(apply(abs(cqf.sim.qr[,,1]-qsftrue.qr[,1]),1,mean))  

RMSE.qr[1,1,2]   <- mean(apply(abs(qsf.sim.QR.qr[,,2]-qsftrue.qr[,2]),1,mean))
RMSE.qr[2,1,2]   <- mean(apply(abs(qsf.sim.DR.qr[,,2]-qsftrue.qr[,2]),1,mean))             
RMSE.qr[3,1,2]   <- 100/1000 * (RMSE.qr[2,1,2]/RMSE.qr[1,1,2])
RMSE.qr[4,1,2]   <- mean(apply(abs(cqf.sim.qr[,,2]-qsftrue.qr[,2]),1,mean))  

RMSE.qr[1,1,3]   <- mean(apply(abs(qsf.sim.QR.qr[,,3]-qsftrue.qr[,3]),1,mean))
RMSE.qr[2,1,3]   <- mean(apply(abs(qsf.sim.DR.qr[,,3]-qsftrue.qr[,3]),1,mean))             
RMSE.qr[3,1,3]   <- 100/1000 * (RMSE.qr[2,1,3]/RMSE.qr[1,1,3])
RMSE.qr[4,1,3]   <- mean(apply(abs(cqf.sim.qr[,,3]-qsftrue.qr[,3]),1,mean)) 


RMSE.qr[1,2,1]   <- mean(sqrt(apply((qsf.sim.QR.qr[,,1]-qsftrue.qr[,1])^2, 1, mean)))
RMSE.qr[2,2,1]   <- mean(sqrt(apply((qsf.sim.DR.qr[,,1]-qsftrue.qr[,1])^2, 1, mean)))
RMSE.qr[3,2,1]   <- 100/1000 * (RMSE.qr[2,2,1]/RMSE.qr[1,2,1])
RMSE.qr[4,2,1]   <- mean(sqrt(apply((cqf.sim.qr[,,1]-qsftrue.qr[,1])^2, 1, mean)))  

RMSE.qr[1,2,2]   <- mean(sqrt(apply((qsf.sim.QR.qr[,,2]-qsftrue.qr[,2])^2, 1, mean)))
RMSE.qr[2,2,2]   <- mean(sqrt(apply((qsf.sim.DR.qr[,,2]-qsftrue.qr[,2])^2, 1, mean)))
RMSE.qr[3,2,2]   <- 100/1000 * (RMSE.qr[2,2,2]/RMSE.qr[1,2,2])
RMSE.qr[4,2,2]   <- mean(sqrt(apply((cqf.sim.qr[,,2]-qsftrue.qr[,2])^2, 1, mean)))  

RMSE.qr[1,2,3]   <- mean(sqrt(apply((qsf.sim.QR.qr[,,3]-qsftrue.qr[,3])^2, 1, mean)))
RMSE.qr[2,2,3]   <- mean(sqrt(apply((qsf.sim.DR.qr[,,3]-qsftrue.qr[,3])^2, 1, mean)))
RMSE.qr[3,2,3]   <- 100/1000 * (RMSE.qr[2,2,3]/RMSE.qr[1,2,3])
RMSE.qr[4,2,3]   <- mean(sqrt(apply((cqf.sim.qr[,,3]-qsftrue.qr[,3])^2, 1, mean)))  


RMSE.qr[1,3,1]   <- max(apply(abs(qsf.sim.QR.qr[,,1]-qsftrue.qr[,1]),1,mean))
RMSE.qr[2,3,1]   <- max(apply(abs(qsf.sim.DR.qr[,,1]-qsftrue.qr[,1]),1,mean))             
RMSE.qr[3,3,1]   <- 100/1000 * (RMSE.qr[2,3,1]/RMSE.qr[1,3,1])
RMSE.qr[4,3,1]   <- max(apply(abs(cqf.sim.qr[,,1]-qsftrue.qr[,1]),1,mean))  

RMSE.qr[1,3,2]   <- max(apply(abs(qsf.sim.QR.qr[,,2]-qsftrue.qr[,2]),1,mean))
RMSE.qr[2,3,2]   <- max(apply(abs(qsf.sim.DR.qr[,,2]-qsftrue.qr[,2]),1,mean))             
RMSE.qr[3,3,2]   <- 100/1000 * (RMSE.qr[2,3,2]/RMSE.qr[1,3,2])
RMSE.qr[4,3,2]   <- max(apply(abs(cqf.sim.qr[,,2]-qsftrue.qr[,2]),1,mean))  

RMSE.qr[1,3,3]   <- max(apply(abs(qsf.sim.QR.qr[,,3]-qsftrue.qr[,3]),1,mean))
RMSE.qr[2,3,3]   <- max(apply(abs(qsf.sim.DR.qr[,,3]-qsftrue.qr[,3]),1,mean))             
RMSE.qr[3,3,3]   <- 100/1000 * (RMSE.qr[2,3,3]/RMSE.qr[1,3,3])
RMSE.qr[4,3,3]   <- max(apply(abs(cqf.sim.qr[,,3]-qsftrue.qr[,3]),1,mean))  


# RMSE.dr
RMSE.dr[1,1,1]   <- mean(apply(abs(qsf.sim.QR.dr[,,1]-qsftrue.dr[,1]),1,mean))
RMSE.dr[2,1,1]   <- mean(apply(abs(qsf.sim.DR.dr[,,1]-qsftrue.dr[,1]),1,mean))            
RMSE.dr[3,1,1]   <- 100/1000 * (RMSE.dr[2,1,1]/RMSE.dr[1,1,1])
RMSE.dr[4,1,1]   <- mean(apply(abs(cqf.sim.dr[,,1]-qsftrue.dr[,1]),1,mean))  

RMSE.dr[1,1,2]   <- mean(apply(abs(qsf.sim.QR.dr[,,2]-qsftrue.dr[,2]),1,mean))
RMSE.dr[2,1,2]   <- mean(apply(abs(qsf.sim.DR.dr[,,2]-qsftrue.dr[,2]),1,mean))             
RMSE.dr[3,1,2]   <- 100/1000 * (RMSE.dr[2,1,2]/RMSE.dr[1,1,2])
RMSE.dr[4,1,2]   <- mean(apply(abs(cqf.sim.dr[,,2]-qsftrue.dr[,2]),1,mean))  

RMSE.dr[1,1,3]   <- mean(apply(abs(qsf.sim.QR.dr[,,3]-qsftrue.dr[,3]),1,mean))
RMSE.dr[2,1,3]   <- mean(apply(abs(qsf.sim.DR.dr[,,3]-qsftrue.dr[,3]),1,mean))             
RMSE.dr[3,1,3]   <- 100/1000 * (RMSE.dr[2,1,3]/RMSE.dr[1,1,3])
RMSE.dr[4,1,3]   <- mean(apply(abs(cqf.sim.dr[,,3]-qsftrue.dr[,3]),1,mean))  


RMSE.dr[1,2,1]   <- mean(sqrt(apply((qsf.sim.QR.dr[,,1]-qsftrue.dr[,1])^2, 1, mean)))
RMSE.dr[2,2,1]   <- mean(sqrt(apply((qsf.sim.DR.dr[,,1]-qsftrue.dr[,1])^2, 1, mean)))
RMSE.dr[3,2,1]   <- 100/1000 * (RMSE.dr[2,2,1]/RMSE.dr[1,2,1])
RMSE.dr[4,2,1]   <- mean(sqrt(apply((cqf.sim.dr[,,1]-qsftrue.dr[,1])^2, 1, mean)))  

RMSE.dr[1,2,2]   <- mean(sqrt(apply((qsf.sim.QR.dr[,,2]-qsftrue.dr[,2])^2, 1, mean)))
RMSE.dr[2,2,2]   <- mean(sqrt(apply((qsf.sim.DR.dr[,,2]-qsftrue.dr[,2])^2, 1, mean)))
RMSE.dr[3,2,2]   <- 100/1000 * (RMSE.dr[2,2,2]/RMSE.dr[1,2,2])
RMSE.dr[4,2,2]   <- mean(sqrt(apply((cqf.sim.dr[,,2]-qsftrue.dr[,2])^2, 1, mean)))  

RMSE.dr[1,2,3]   <- mean(sqrt(apply((qsf.sim.QR.dr[,,3]-qsftrue.dr[,3])^2, 1, mean)))
RMSE.dr[2,2,3]   <- mean(sqrt(apply((qsf.sim.DR.dr[,,3]-qsftrue.dr[,3])^2, 1, mean)))
RMSE.dr[3,2,3]   <- 100/1000 * (RMSE.dr[2,2,3]/RMSE.dr[1,2,3])
RMSE.dr[4,2,3]   <- mean(sqrt(apply((cqf.sim.dr[,,3]-qsftrue.dr[,3])^2, 1, mean)))  


RMSE.dr[1,3,1]   <- max(apply(abs(qsf.sim.QR.dr[,,1]-qsftrue.dr[,1]),1,mean))
RMSE.dr[2,3,1]   <- max(apply(abs(qsf.sim.DR.dr[,,1]-qsftrue.dr[,1]),1,mean))             
RMSE.dr[3,3,1]   <- 100/1000 * (RMSE.dr[2,3,1]/RMSE.dr[1,3,1])
RMSE.dr[4,3,1]   <- max(apply(abs(cqf.sim.dr[,,1]-qsftrue.dr[,1]),1,mean))  

RMSE.dr[1,3,2]   <- max(apply(abs(qsf.sim.QR.dr[,,2]-qsftrue.dr[,2]),1,mean))
RMSE.dr[2,3,2]   <- max(apply(abs(qsf.sim.DR.dr[,,2]-qsftrue.dr[,2]),1,mean))             
RMSE.dr[3,3,2]   <- 100/1000 * (RMSE.dr[2,3,2]/RMSE.dr[1,3,2])
RMSE.dr[4,3,2]   <- max(apply(abs(cqf.sim.dr[,,2]-qsftrue.dr[,2]),1,mean))  

RMSE.dr[1,3,3]   <- max(apply(abs(qsf.sim.QR.dr[,,3]-qsftrue.dr[,3]),1,mean))
RMSE.dr[2,3,3]   <- max(apply(abs(qsf.sim.DR.dr[,,3]-qsftrue.dr[,3]),1,mean))             
RMSE.dr[3,3,3]   <- 100/1000 * (RMSE.dr[2,3,3]/RMSE.dr[1,3,3])
RMSE.dr[4,3,3]   <- max(apply(abs(cqf.sim.dr[,,3]-qsftrue.dr[,3]),1,mean))  


# RMSE.ASF.loc
RMSE.ASF.loc[1,1]   <- mean(apply(abs(asf.sim.QR.loc-asftrue.loc),1,mean))
RMSE.ASF.loc[2,1]   <- mean(apply(abs(asf.sim.DR.loc-asftrue.loc),1,mean))             
RMSE.ASF.loc[3,1]   <- 100/1000 * (RMSE.ASF.loc[2,1]/RMSE.ASF.loc[1,1])
RMSE.ASF.loc[4,1]   <- mean(apply(abs(cmf.sim.loc-asftrue.loc),1,mean))  


RMSE.ASF.loc[1,2]   <- mean(sqrt(apply((asf.sim.QR.loc-asftrue.loc)^2, 1, mean)))
RMSE.ASF.loc[2,2]   <- mean(sqrt(apply((asf.sim.DR.loc-asftrue.loc)^2, 1, mean)))
RMSE.ASF.loc[3,2]   <- 100/1000 * (RMSE.ASF.loc[2,2]/RMSE.ASF.loc[1,2])
RMSE.ASF.loc[4,2]   <- mean(sqrt(apply((cmf.sim.loc-asftrue.loc)^2, 1, mean)))  


RMSE.ASF.loc[1,3]   <- max(apply(abs(asf.sim.QR.loc-asftrue.loc),1,mean))
RMSE.ASF.loc[2,3]   <- max(apply(abs(asf.sim.DR.loc-asftrue.loc),1,mean))             
RMSE.ASF.loc[3,3]   <- 100/1000 * (RMSE.ASF.loc[2,3]/RMSE.ASF.loc[1,3])
RMSE.ASF.loc[4,3]   <- max(apply(abs(cmf.sim.loc-asftrue.loc),1,mean))  


# RMSE.ASF.qr
RMSE.ASF.qr[1,1]   <- mean(apply(abs(asf.sim.QR.qr-asftrue.qr),1,mean))
RMSE.ASF.qr[2,1]   <- mean(apply(abs(asf.sim.DR.qr-asftrue.qr),1,mean))             
RMSE.ASF.qr[3,1]   <- 100/1000 * (RMSE.ASF.qr[2,1]/RMSE.ASF.qr[1,1])
RMSE.ASF.qr[4,1]   <- mean(apply(abs(cmf.sim.qr-asftrue.qr),1,mean))  


RMSE.ASF.qr[1,2]   <- mean(sqrt(apply((asf.sim.QR.qr-asftrue.qr)^2, 1, mean)))
RMSE.ASF.qr[2,2]   <- mean(sqrt(apply((asf.sim.DR.qr-asftrue.qr)^2, 1, mean)))
RMSE.ASF.qr[3,2]   <- 100/1000 * (RMSE.ASF.qr[2,2]/RMSE.ASF.qr[1,2])
RMSE.ASF.qr[4,2]   <- mean(sqrt(apply((cmf.sim.qr-asftrue.qr)^2, 1, mean)))  


RMSE.ASF.qr[1,3]   <- max(apply(abs(asf.sim.QR.qr-asftrue.qr),1,mean))
RMSE.ASF.qr[2,3]   <- max(apply(abs(asf.sim.DR.qr-asftrue.qr),1,mean))             
RMSE.ASF.qr[3,3]   <- 100/1000 * (RMSE.ASF.qr[2,3]/RMSE.ASF.qr[1,3])
RMSE.ASF.qr[4,3]   <- max(apply(abs(cmf.sim.qr-asftrue.qr),1,mean)) 



# RMSE.ASF.dr
RMSE.ASF.dr[1,1]   <- mean(apply(abs(asf.sim.QR.dr-asftrue.dr),1,mean))
RMSE.ASF.dr[2,1]   <- mean(apply(abs(asf.sim.DR.dr-asftrue.dr),1,mean))            
RMSE.ASF.dr[3,1]   <- 100/1000 * (RMSE.ASF.dr[2,1]/RMSE.ASF.dr[1,1])
RMSE.ASF.dr[4,1]   <- mean(apply(abs(cmf.sim.dr-asftrue.dr),1,mean))  


RMSE.ASF.dr[1,2]   <- mean(sqrt(apply((asf.sim.QR.dr-asftrue.dr)^2, 1, mean)))
RMSE.ASF.dr[2,2]   <- mean(sqrt(apply((asf.sim.DR.dr-asftrue.dr)^2, 1, mean)))
RMSE.ASF.dr[3,2]   <- 100/1000 * (RMSE.ASF.dr[2,2]/RMSE.ASF.dr[1,2])
RMSE.ASF.dr[4,2]   <- mean(sqrt(apply((cmf.sim.dr-asftrue.dr)^2, 1, mean)))  


RMSE.ASF.dr[1,3]   <- max(apply(abs(asf.sim.QR.dr-asftrue.dr),1,mean))
RMSE.ASF.dr[2,3]   <- max(apply(abs(asf.sim.DR.dr-asftrue.dr),1,mean))             
RMSE.ASF.dr[3,3]   <- 100/1000 * (RMSE.ASF.dr[2,3]/RMSE.ASF.dr[1,3])
RMSE.ASF.dr[4,3]   <- max(apply(abs(cmf.sim.dr-asftrue.dr),1,mean))  


results_sim                 <- list()

results_sim$good            <- good
results_sim$nsim            <- nsim
results_sim$rho             <- rho
results_sim$link            <- link
results_sim$ntaus1          <- ntaus1
results_sim$ntaus2          <- ntaus2

results_sim$tausqsf         <- tausqsf
results_sim$Xgridsim        <- Xgridsim
results_sim$RMSE.loc        <- RMSE.loc
results_sim$RMSE.qr         <- RMSE.qr
results_sim$RMSE.dr         <- RMSE.dr
results_sim$RMSE.ASF.loc    <- RMSE.ASF.loc
results_sim$RMSE.ASF.qr     <- RMSE.ASF.qr
results_sim$RMSE.ASF.dr     <- RMSE.ASF.dr

results_sim$QR.loc          <- qsf.sim.QR.loc
results_sim$DR.loc          <- qsf.sim.DR.loc
results_sim$asf.QR.loc      <- asf.sim.QR.loc
results_sim$asf.DR.loc      <- asf.sim.DR.loc
results_sim$cqf.loc         <- cqf.sim.loc
results_sim$cmf.loc         <- cmf.sim.loc
results_sim$qsftrue.loc     <- qsftrue.loc
results_sim$asftrue.loc     <- asftrue.loc

results_sim$QR.qr           <- qsf.sim.QR.qr
results_sim$DR.qr           <- qsf.sim.DR.qr
results_sim$asf.QR.qr       <- asf.sim.QR.qr
results_sim$asf.DR.qr       <- asf.sim.DR.qr
results_sim$cqf.qr          <- cqf.sim.qr
results_sim$cmf.qr          <- cmf.sim.qr
results_sim$qsftrue.qr      <- qsftrue.qr
results_sim$asftrue.qr      <- asftrue.qr

results_sim$QR.dr           <- qsf.sim.QR.dr
results_sim$DR.dr           <- qsf.sim.DR.dr
results_sim$asf.QR.dr       <- asf.sim.QR.dr
results_sim$asf.DR.dr       <- asf.sim.DR.dr
results_sim$cqf.dr          <- cqf.sim.dr
results_sim$cmf.dr          <- cmf.sim.dr
results_sim$qsftrue.dr      <- qsftrue.dr
results_sim$asftrue.dr      <- asftrue.qr

save(results_sim, file = paste("SimResults_",good,"_",nsim,"_",link,"_",rho,"_ng",ntaus,"_nx",ngridX,".RData", sep="") )
