# A simple routine for distribution regression.
# Trimming done externally for control function estimation.
# Incorporates speedglm().
# 'gridY': evaluation points for CDF.

# This version: June 2019.

distrfun <- function(Y, X, gridY, link = "logit", weights = NULL, smplval = FALSE, maxit = 300){
  
  # DR coefs estimation
  options(warn=-1)
  nobs        <- length(Y)
  ngy         <- length(gridY)
  indexY      <- sapply( 1:ngy, function(m) 1*( Y <= gridY[m] ) )     # Constructing matrix of indicators;
  colnames(X) <- paste("X",1:ncol(X),sep = "")
  coefs0      <- NULL
  try( coefs0 <- sapply( 1:ngy, function(m) speedglm(as.formula(paste("indexY[ ,m]~",paste(paste("X",1:ncol(X),sep=""),collapse="+"))), data = data.frame(cbind(indexY[,m],X)), family = binomial(link = link), weights = weights, control = list(maxit = maxit) )$coef ) )
  if(length(coefs0)==0){ 
    coefs0      <- sapply( 1:ngy, function(m) glm(indexY[ ,m] ~ X, family = binomial(link = link), weights = weights, control = list(maxit = maxit) )$coef ) 
  }
  if(smplval == TRUE){
    # Interpolate at sample values
    x         <- cbind(1,X)
    cdf       <- matrix(0, nobs, nobs)  
    
    # DR coefs approximation
    bfunc     <- list()    
    nx        <- ncol(x)          
    coefs     <- matrix(0, nx, nobs)   

    # Linear interpolation of dr coefficients
    for(j in 1:nx){
          bfunc[[j]] <- approxfun( gridY, coefs0[j,], rule = 2 )         
          coefs[j,]  <- bfunc[[j]](Y)
          }
        
    # Estimates
    if(link == "probit") cdf <- sapply( 1:nobs, function(m) pnorm( coefs[,m] %*% t(x) ) ) 
    if(link == "logit")  cdf <- sapply( 1:nobs, function(m) plogis(coefs[,m] %*% t(x) ) ) 

    # Extrapolation of distribution function values
    cdfn <- diag(cdf) 
    ifelse(cdfn<(1-1e-9), cdfn <- cdfn, cdfn[which(cdfn>(1-1e-9))] <- max(cdfn[-which(cdfn>(1-1e-9))]))
    ifelse(cdfn>(0+1e-9), cdfn <- cdfn, cdfn[which(cdfn<(0+1e-9))] <- min(cdfn[-which(cdfn<(0+1e-9))]))
  }
  
  ifelse(smplval == FALSE, ans <- list(coefs0 = coefs0, gridY = gridY), ans <- list(coefs = coefs, coefs0 = coefs0, cdfn = cdfn, gridY = gridY) )

  return(ans)
  
  }
