# This is the main script for the empirical illustration the manuscript.
#
# This version : June 2019


# Settings
#rm(list=ls())    # This removes everything!

# Load relevant libraries
library(foreign);
library(quantreg);
library(splines);
library(doParallel)
library(doRNG)
library(speedglm)
registerDoParallel(makeCluster(detectCores()))			 
getDoParWorkers()                                        # Sanity check


# Set working directory, load relevant functions
#--------------------------------------------------------------------
setwd("")


# Source relevant functions
source("BinLib.R")
source("mainfuncs.R")
source("rearrangement.R")
source("drfunc.R")


#=========================================================
#          
#                       General settings  
#
#=========================================================

# General
tmp           <- NULL
Qreg          <- TRUE                     # Whether the QR estimator should be implemented
Dreg          <- TRUE                     # Whether the DR estimator should be implemented
smallgX       <- NULL
#LS            <- NULL

link          <- "logit"                  # Link function for distribution regression
rSS           <- 0                        # Rearranged DR-based DSF: set rSS = 0. 
eps           <- 0.01 
trim          <- c(0,1)                   # No trimming
ngridY        <- 599
ntaus1        <- 599;                      # number of QR grid points in first stage
ntaus2        <- 599;                      # number of QR grid points in second stage
qY            <- seq(0, 1, length.out = ngridY)
taus1         <- seq(eps, 1-eps, length.out = ntaus1)   # Grid second stage           
taus2         <- seq(eps, 1-eps, length.out = ntaus2);  # Grid first stage     
se            <- 1
ngy           <- 15
qYbar         <- seq(.1, .9, length.out = ngy)

# Bootstrap
B             <- 199                    # Number of bootstrap replications
seed          <- 999                    # Fix the seed for replication
alphab        <- .1                     # Confidence level 1-alpha

# CV
polyCV        <- 5                      # Powers in CV 



# =========================================================
#          
#                       Implementation 
#
# =========================================================

dat <- read.dta("engelpool.dta");
attach(dat);

X  	          <- logexp;
Z             <- logwages;
sample        <- "pool"
#wboots        <- matrix(rexp(length(X) * B), length(X), B)
#save(wboots, file = paste("weights_empappli.RData",sep="") )
load("weights_empappli.RData")    # Weights draw used for inference 
R             <- nkids;
R.sens        <- nkids;


sensitivity   <- NULL

# Version 1: FOOD
#--------------------------------------------------------------------

good          <- "Food"
Y             <- food;

integration   <- "Smplave"            # "Smplave": sample average; "simu": simulation.
sensitivity   <- NULL

ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf)

ngridX        <- 3;
#gridX         <- seq( quantile(X, .1), quantile(X,.9), length.out = ngridX)
gridX         <- quantile(X, seq(.1, .9, length.out = ngridX))

source("Empirical_Application_CFNSV_par.R")

sensitivity   <- TRUE

ngridX        <- 5;
#gridX         <- seq( quantile(X, .1), quantile(X,.9), length.out = ngridX)
gridX         <- quantile(X, seq(.1, .9, length.out = ngridX))

source("Empirical_Application_CFNSV_par.R")

sensitivity   <- NULL

ntausqsf      <- 5;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf)

source("Empirical_Application_CFNSV_par.R")

smallgX       <- TRUE
#gridX         <- seq( quantile(X, .15), quantile(X,.85), length.out = ngridX)
gridX         <- quantile(X, seq(.15, .85, length.out = ngridX))

source("Empirical_Application_CFNSV_par.R")

smallgX       <- NULL

ngridX        <- 7;
#gridX         <- seq( quantile(X, .1), quantile(X,.9), length.out = ngridX)
gridX         <- quantile(X, seq(.1, .9, length.out = ngridX))

#source("Empirical_Application_CFNSV_par.R")

ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf)

source("Empirical_Application_CFNSV_par.R")


integration   <- "simu"                   # "Smplave": sample average; "simu": simulation.
se            <- NULL
ngridX        <- 5;
#gridX         <- seq( quantile(X, .1), quantile(X,.9), length.out = ngridX)
gridX         <- quantile(X, seq(.1, .9, length.out = ngridX))

source("Empirical_Application_CFNSV_par.R")

se            <- 1



# Version 2: LEISURE
#--------------------------------------------------------------------

good          <- "Leisure"
Y             <- leisure;

integration   <- "Smplave"            # "Smplave": sample average; "simu": simulation.
sensitivity   <- NULL

ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf)

ngridX        <- 3;
#gridX         <- seq( quantile(X, .1), quantile(X,.9), length.out = ngridX)
gridX         <- quantile(X, seq(.1, .9, length.out = ngridX))

source("Empirical_Application_CFNSV_par.R")

sensitivity   <- TRUE

ngridX        <- 5;
#gridX         <- seq( quantile(X, .1), quantile(X,.9), length.out = ngridX)
gridX         <- quantile(X, seq(.1, .9, length.out = ngridX))

source("Empirical_Application_CFNSV_par.R")

sensitivity   <- NULL

ntausqsf      <- 5;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf)

source("Empirical_Application_CFNSV_par.R")

smallgX       <- TRUE
#gridX         <- seq( quantile(X, .15), quantile(X,.85), length.out = ngridX)
gridX         <- quantile(X, seq(.15, .85, length.out = ngridX))

source("Empirical_Application_CFNSV_par.R")

smallgX       <- NULL

ngridX        <- 7;
#gridX         <- seq( quantile(X, .1), quantile(X,.9), length.out = ngridX)
gridX         <- quantile(X, seq(.1, .9, length.out = ngridX))

#source("Empirical_Application_CFNSV_par.R")

ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf)

source("Empirical_Application_CFNSV_par.R")


integration   <- "simu"                   # "Smplave": sample average; "simu": simulation.
se            <- NULL
ngridX        <- 5;
#gridX         <- seq( quantile(X, .1), quantile(X,.9), length.out = ngridX)
gridX         <- quantile(X, seq(.1, .9, length.out = ngridX))

source("Empirical_Application_CFNSV_par.R")



detach(dat)

