# This is the file that compares ASF estimation via OLS and via QR.
# Generates Figure 5 in the supplementary material.

# This version : June 2019.


# Settings
#rm(list=ls())

library(foreign);
library(quantreg);
library(ggplot2);
library(doParallel)
library(doRNG)
registerDoParallel(makeCluster(detectCores()))			 



# Set working directory, load relevant functions
#--------------------------------------------------------------------
setwd("")


source("BinLib.R")
source("mainfuncs.R")
source("rearrangement.R")
source("drfunc.R")
source("sfuncsplot.R")


#=========================================================
#          
#                       General settings  
#
#=========================================================

# General
#link          <- "logit"                  # Link function for distribution regression
#rSS           <- 0                        # Rearranged DR-based DSF = 0. 
eps           <- 0.01
trim          <- c(0,1)
ngridY        <- 599
ntaus1        <- 599;                      # number of QR grid points in first stage
ntaus2        <- 599;                      # number of QR grid points in second stage
#qY            <- seq(eps, 1-eps, length.out = ngridY)
qY            <- seq(0, 1, length.out = ngridY)
taus1         <- seq(eps, 1-eps, length.out = ntaus1)              
taus2         <- seq(eps, 1-eps, length.out = ntaus2);
se            <- 1
ngy           <- 15
qYbar         <- NULL

# Bootstrap
B             <- 199                      # Number of bootstrap replications
seed          <- 999
alphab        <- .1                      # Confidence level 1-alpha



# =========================================================
#          
#                       Implementation 
#
# =========================================================


dat <- read.dta("engelpool.dta");
attach(dat);


good          <- "Leisure"
Y             <- leisure
X  	          <- logexp
Z             <- logwages
R             <- nkids
sample        <- "pool"

#wboots        <- matrix(rexp(length(X) * B), length(X), B)
#save(wboots, file = paste("weights_empappli.RData",sep="") )
load("weights_empappli.RData")

ngridX         <- 5;
#gridX          <- seq( quantile(X, .1), quantile(X,.9), length.out = ngridX)
gridX         <- quantile(X, seq(.1, .9, length.out = ngridX))

# Plot settings
nxx           <- 15
ngy           <- 15
pY            <- seq(.1,.9,length.out = ngy)
#ylab          <- "Average Structural Function"
yy            <- quantile(Y,pY)
ylab.asf      <- "Average Structural Function"
ylim.asf      <- c(0,0.35)
ntausqsf      <- 3


# ASF estimation
# Run the next line to perform OLS estimation...
# source("Empirical_Application_ASFLS.R") 
# ... and comment the next line off.
load("LSASFResultspm_Leisure_pool_ngFS599_ngSS599_nx5t0_qY0.RData")
resultsLS <- results

load("EstimationResultspm_Leisure_pool_Smplave_logit_ngFS599_ngSS599_ntqsf3_nx5t0_qY0.RData")
fplotpm     <- sfuncsplot(results = results, ngx = nxx, yy = yy, resultsLS = resultsLS)

xx            <- seq(results$DR$gridX[1],results$DR$gridX[ngridX],length.out = nxx)
xlim          <- c(min(results$DR$gridX)-.05, max(results$DR$gridX)+.05)

ASFpm      <- data.frame(Xgrid=xx, colors=factor(rep(c(1,2), each=nxx)), mean=c(fplotpm$asfQRfunx,fplotpm$asfLSfunx), lwr=c(fplotpm$asfQRfunx.lbw,fplotpm$asfLSfunx.lbw), upr=c(fplotpm$asfQRfunx.ubw,fplotpm$asfLSfunx.ubw) )

pASFpm <- ggplot(ASFpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.asf) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=ASFpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=ASFpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_line(aes(color=colors),size=1.5) + ylab(ylab.asf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                                                             panel.grid.major=element_blank(), 
                                                                                             panel.background = element_rect(fill = 'white', colour = 'black'),
                                                                                             axis.title.y = element_text(face="bold"),
                                                                                             axis.text.y  = element_text(face="bold",colour=1),
                                                                                             axis.title.x = element_text(face="bold"),
                                                                                             axis.text.x  = element_text(face="bold",colour=1),
                                                                                             legend.position="none")

pASFpm


#pdf( paste("ASFcomparison_Leisure.pdf", sep="") )
pdf( paste("Figure5_Leisure_SM.pdf", sep="") )
pASFpm
dev.off()

rm(ls="results")


###################################

good           <- "Food"
Y              <- food


# Run the next line to perform OLS estimation...
#source("Empirical_Application_ASFLS.R")

# ... and comment the next line off.
load("LSASFResultspm_Food_pool_ngFS599_ngSS599_nx5t0_qY0.RData")
resultsLS <- results

load("EstimationResultspm_Food_pool_Smplave_logit_ngFS599_ngSS599_ntqsf3_nx5t0_qY0.RData")
fplotpm     <- sfuncsplot(results = results, ngx = nxx, yy = yy, resultsLS = resultsLS)

xx            <- seq(results$DR$gridX[1],results$DR$gridX[ngridX],length.out = nxx)
xlim          <- c(min(results$DR$gridX)-.05, max(results$DR$gridX)+.05)

ASFpm      <- data.frame(Xgrid=xx, colors=factor(rep(c(1,2), each=nxx)), mean=c(fplotpm$asfQRfunx,fplotpm$asfLSfunx), lwr=c(fplotpm$asfQRfunx.lbw,fplotpm$asfLSfunx.lbw), upr=c(fplotpm$asfQRfunx.ubw,fplotpm$asfLSfunx.ubw) )

pASFpm <- ggplot(ASFpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.asf) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=ASFpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=ASFpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_line(aes(color=colors),size=1.5) + ylab(ylab.asf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                                                             panel.grid.major=element_blank(), 
                                                                                             panel.background = element_rect(fill = 'white', colour = 'black'),
                                                                                             axis.title.y = element_text(face="bold"),
                                                                                             axis.text.y  = element_text(face="bold",colour=1),
                                                                                             axis.title.x = element_text(face="bold"),
                                                                                             axis.text.x  = element_text(face="bold",colour=1),
                                                                                             legend.position="none")

pASFpm


#pdf( paste("ASFcomparison_Food.pdf", sep="") )
pdf( paste("Figure5_Food_SM.pdf", sep="") )
pASFpm
dev.off()

rm(ls="results")

