# This file produces all figures in the manuscript for food application.
# Except Figures 5, 7 and 9 in supplementary material.

# This version: June 2019.

#rm(list = ls())

library(ggplot2);
library(foreign);

# Set working directory here
setwd("")

source("sfuncsplot.R")
source("sfuncsplot_polys.R")


# ==============================================================================
#                        Figures
# ==============================================================================

good          <- "Food"
# General settings : fixed across plots
ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf) 
ntaus1        <- 599;                    # number of grid points in first stage
ntaus2        <- 599;                    # number of grid points in second stage
trim          <- NULL
link          <- "logit"                  # Link function for distribution regression
integration   <- "Smplave"
nxx           <- 15
ngy           <- 15
pY            <- seq(.1,.9,length.out = ngy)
ylab.qsf      <- "Quantile Structural Function"
ylab.asf      <- "Average Structural Function"
ylab.dsf      <- "Distribution Structural Function"
ylim.qsf      <- c(0,0.4)
ylim.asf      <- c(0.05,0.35)
ylim.dsf      <- c(-.01,1.01)
xlab.dsf      <- paste(good, " Expenditure Share", sep="")
ylim.qsf5     <- c(0,0.5)
ylim.qsf      <- c(0,0.5)



# ========================================
#
#        FULL SAMPLE
#
# ========================================

sample        <- "pool"
dat <- read.dta("engelpool.dta");
attach(dat);
#Y             <- food
Y             <- food
yy            <- quantile(Y,pY)
xlim.dsf      <- c(min(yy)-.025, max(yy)+.025)


# ngridX = 3; ntqsf = 3
# ------------------------------------

ngridX        <- 3
ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf) 


load("EstimationResultspm_Food_pool_Smplave_logit_ngFS599_ngSS599_ntqsf3_nx3t0_qY0.RData")
fplotpm    <- sfuncsplot(results = results, ngx = nxx, yy = yy)

xx         <- seq(results$DR$gridX[1],results$DR$gridX[ngridX],length.out = nxx)
xlim       <- c(min(results$DR$gridX)-.05, max(results$DR$gridX)+.05)

QSFQRpm    <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,1,3), each=nxx)), mean = fplotpm$qsfQRfunx, lwr = fplotpm$qsfQRfunx.lbw, upr=fplotpm$qsfQRfunx.ubw )

QSFDRpm    <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,1,3), each=nxx)), mean = fplotpm$qsfDRfunx, lwr = fplotpm$qsfDRfunx.lbw, upr=fplotpm$qsfDRfunx.ubw )

DSFQRpm    <- data.frame(Ygrid = rep(yy,ngridX), colors = factor(rep(c(2,1,3), each=ngy)), mean = fplotpm$dsfQRfunx, lwr = fplotpm$dsfQRfunx.lbw, upr=fplotpm$dsfQRfunx.ubw )

DSFDRpm    <- data.frame(Ygrid = rep(yy,ngridX), colors = factor(rep(c(2,1,3), each=ngy)), mean = fplotpm$dsfDRfunx, lwr = fplotpm$dsfDRfunx.lbw, upr=fplotpm$dsfDRfunx.ubw )


pQSFQRpm <- ggplot(QSFQRpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFQRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFQRpm

pQSFDRpm <- ggplot(QSFDRpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFDRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFDRpm

pDSFQRpm <- ggplot(DSFQRpm, aes(x=Ygrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.dsf) + xlim(xlim.dsf) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=DSFQRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=DSFQRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=DSFQRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  geom_line(aes(color=colors)) + 
  ylab(ylab.dsf) + xlab(xlab.dsf) + theme(panel.grid.minor=element_blank(), 
                                                        panel.grid.major=element_blank(), 
                                                        panel.background = element_rect(fill = 'white', colour = 'black'),
                                                        axis.title.y = element_text(face="bold"),
                                                        axis.text.y  = element_text(face="bold",colour=1),
                                                        axis.title.x = element_text(face="bold"),
                                                        axis.text.x  = element_text(face="bold",colour=1),
                                                        legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pDSFQRpm

pDSFDRpm <- ggplot(DSFDRpm, aes(x=Ygrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.dsf) + xlim(xlim.dsf) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=DSFDRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=DSFDRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=DSFDRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  geom_line(aes(color=colors)) + 
  ylab(ylab.dsf) + xlab(xlab.dsf) + theme(panel.grid.minor=element_blank(), 
                                                        panel.grid.major=element_blank(), 
                                                        panel.background = element_rect(fill = 'white', colour = 'black'),
                                                        axis.title.y = element_text(face="bold"),
                                                        axis.text.y  = element_text(face="bold",colour=1),
                                                        axis.title.x = element_text(face="bold"),
                                                        axis.text.x  = element_text(face="bold",colour=1),
                                                        legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pDSFDRpm


#pdf( paste("BootQSFQRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure1A_QR_SM.pdf", sep="") )
pQSFQRpm
dev.off()

#pdf( paste("BootQSFDRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure1A_DR_SM.pdf", sep="") )
pQSFDRpm
dev.off()

#pdf( paste("BootDSFQRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure3A_QR.pdf", sep="") )
pDSFQRpm
dev.off()

#pdf( paste("BootDSFDRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure3A_DR.pdf", sep="") )
pDSFDRpm
dev.off()


rm(ls="results")



# ngridX = 5; ntqsf = 3
# ------------------------------------

ngridX        <- 5
ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf) 

load("EstimationResultspm_Food_pool_Smplave_logit_ngFS599_ngSS599_ntqsf3_nx5t0_qY0.RData")
fplotpm       <- sfuncsplot(results = results, ngx = nxx, yy = yy)

xx            <- seq(results$DR$gridX[1],results$DR$gridX[ngridX],length.out = nxx)
xlim          <- c(min(results$DR$gridX)-.05, max(results$DR$gridX)+.05)


QSFQRpm    <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,1,3), each=nxx)), mean = fplotpm$qsfQRfunx, lwr = fplotpm$qsfQRfunx.lbw, upr = fplotpm$qsfQRfunx.ubw )

QSFDRpm    <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,1,3), each=nxx)), mean = fplotpm$qsfDRfunx, lwr = fplotpm$qsfDRfunx.lbw, upr = fplotpm$qsfDRfunx.ubw )

ASFpm      <- data.frame(Xgrid=xx, colors=factor(rep(c(1,2), each=nxx)), mean=c(fplotpm$asfDRfunx,fplotpm$asfQRfunx), lwr=c(fplotpm$asfDRfunx.lbw,fplotpm$asfQRfunx.lbw), upr=c(fplotpm$asfDRfunx.ubw,fplotpm$asfQRfunx.ubw) )

ASFQRpm <- data.frame(Xgrid=xx, colors=rep(1, each=nxx), mean=fplotpm$asfQRfunx, lwr=fplotpm$asfQRfunx.lbw, upr=fplotpm$asfQRfunx.ubw )

ASFDRpm <- data.frame(Xgrid=xx, colors=rep(1, each=nxx), mean=fplotpm$asfDRfunx, lwr=fplotpm$asfDRfunx.lbw, upr=fplotpm$asfDRfunx.ubw )



pQSFQRpm <- ggplot(QSFQRpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFQRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") + 
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFQRpm

pQSFDRpm <- ggplot(QSFDRpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFDRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFDRpm


pASFpm <- ggplot(ASFpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.asf) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=ASFpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=ASFpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_line(aes(color=colors),size=1.5) + ylab(ylab.asf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                                                           panel.grid.major=element_blank(), 
                                                                                           panel.background = element_rect(fill = 'white', colour = 'black'),
                                                                                           axis.title.y = element_text(face="bold"),
                                                                                           axis.text.y  = element_text(face="bold",colour=1),
                                                                                           axis.title.x = element_text(face="bold"),
                                                                                           axis.text.x  = element_text(face="bold",colour=1),
                                                                                           legend.position="none")
pASFpm



#pdf( paste("BootQSFQRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure1A_QR.pdf", sep="") )
pQSFQRpm
dev.off()

#pdf( paste("BootQSFDRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure1A_DR.pdf", sep="") )
pQSFDRpm
dev.off()

#pdf( paste("BootASFpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure2_Left.pdf", sep="") )
pASFpm
dev.off()


rm(ls="results")



# ngridX = 5; ntqsf = 5
# ------------------------------------

ngridX        <- 5
ntausqsf      <- 5;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf) 

load("EstimationResultspm_Food_pool_Smplave_logit_ngFS599_ngSS599_ntqsf5_nx5t0_qY0.RData")
fplotpm       <- sfuncsplot(results = results, ngx = nxx, yy = yy)

xx            <- seq(results$DR$gridX[1],results$DR$gridX[ngridX],length.out = nxx)
xlim          <- c(min(results$DR$gridX)-.05, max(results$DR$gridX)+.05)

load("EstimationResultspmsg_Food_pool_Smplave_logit_ngFS599_ngSS599_ntqsf5_nx5t0_qY0.RData")
fplotpmsg     <- sfuncsplot(results = results, ngx = nxx, yy = yy)

xxsg          <- seq(results$DR$gridX[1],results$DR$gridX[ngridX],length.out = nxx)
xlimsg        <- c(min(results$DR$gridX)-.05, max(results$DR$gridX)+.05)


QSFQRpm    <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,3,1,4,5), each=nxx)), mean = fplotpm$qsfQRfunx, lwr = fplotpm$qsfQRfunx.lbw, upr = fplotpm$qsfQRfunx.ubw )

QSFDRpm    <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,3,1,4,5), each=nxx)), mean = fplotpm$qsfDRfunx, lwr = fplotpm$qsfDRfunx.lbw, upr = fplotpm$qsfDRfunx.ubw )

QSFQRpmsg  <- data.frame(Xgrid = rep(xxsg,ntausqsf), colors = factor(rep(c(2,3,1,4,5), each=nxx)), mean = fplotpmsg$qsfQRfunx, lwr = fplotpmsg$qsfQRfunx.lbw, upr = fplotpmsg$qsfQRfunx.ubw )

QSFDRpmsg  <- data.frame(Xgrid = rep(xxsg,ntausqsf), colors = factor(rep(c(2,3,1,4,5), each=nxx)), mean = fplotpmsg$qsfDRfunx, lwr = fplotpmsg$qsfDRfunx.lbw, upr = fplotpmsg$qsfDRfunx.ubw )


pQSFQRpm <- ggplot(QSFQRpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf5) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFQRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(3*nxx+1):(4*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(4*nxx+1):(5*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFQRpm

pQSFDRpm <- ggplot(QSFDRpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf5) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFDRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(3*nxx+1):(4*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(4*nxx+1):(5*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFDRpm

pQSFQRpmsg <- ggplot(QSFQRpmsg, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf5) + xlim(xlimsg) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFQRpmsg[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpmsg[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpmsg[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpmsg[(3*nxx+1):(4*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpmsg[(4*nxx+1):(5*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") + 
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFQRpmsg

pQSFDRpmsg <- ggplot(QSFDRpmsg, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf5) + xlim(xlimsg) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFDRpmsg[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpmsg[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpmsg[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpmsg[(3*nxx+1):(4*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpmsg[(4*nxx+1):(5*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFDRpmsg


#pdf( paste("BootQSFQRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure3A_QR_SM.pdf", sep="") )
pQSFQRpm
dev.off()

#pdf( paste("BootQSFDRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure3A_DR_SM.pdf", sep="") )
pQSFDRpm
dev.off()


#pdf( paste("BootQSFQRpmsg_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure4A_QR_SM.pdf", sep="") )
pQSFQRpmsg
dev.off()

#pdf( paste("BootQSFDRpmsg_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure4A_DR_SM.pdf", sep="") )
pQSFDRpmsg
dev.off()

rm(ls="results")



# ngridX = 7; ntqsf = 3
# ------------------------------------

ngridX        <- 7
ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf) 

load("EstimationResultspm_Food_pool_Smplave_logit_ngFS599_ngSS599_ntqsf3_nx7t0_qY0.RData")
fplotpm       <- sfuncsplot(results = results, ngx = nxx, yy = yy)

xx            <- seq(results$DR$gridX[1],results$DR$gridX[ngridX],length.out = nxx)
xlim          <- c(min(results$DR$gridX)-.05, max(results$DR$gridX)+.05)


QSFQRpm    <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(3,1,4), each=nxx)), mean = fplotpm$qsfQRfunx, lwr = fplotpm$qsfQRfunx.lbw, upr = fplotpm$qsfQRfunx.ubw )

QSFDRpm    <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(3,1,4), each=nxx)), mean = fplotpm$qsfDRfunx, lwr = fplotpm$qsfDRfunx.lbw, upr = fplotpm$qsfDRfunx.ubw )

pQSFQRpm <- ggplot(QSFQRpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFQRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFQRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFQRpm

pQSFDRpm <- ggplot(QSFDRpm, aes(x=Xgrid, y=mean, ymin=lwr, ymax=upr)) + ylim(ylim.qsf) + xlim(xlim) + 
  geom_ribbon(aes(fill=colors), alpha=0.3) +
  geom_ribbon(color="dark gray",data=QSFDRpm[1:nxx,],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(nxx+1):(2*nxx),],alpha=0,linetype=1,size=.05) +
  geom_ribbon(color="dark gray",data=QSFDRpm[(2*nxx+1):(3*nxx),],alpha=0,linetype=1,size=.05) +
  ylab(ylab.qsf) + xlab("Total Expenditure") + theme(panel.grid.minor=element_blank(), 
                                                     panel.grid.major=element_blank(), 
                                                     panel.background = element_rect(fill = 'white', colour = 'black'),
                                                     axis.title.y = element_text(face="bold"),
                                                     axis.text.y  = element_text(face="bold",colour=1),
                                                     axis.title.x = element_text(face="bold"),
                                                     axis.text.x  = element_text(face="bold",colour=1),
                                                     legend.position="none") +
  geom_line(aes(color=colors),size=1.5) #+ 

pQSFDRpm



#pdf( paste("BootQSFQRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure2A_QR_SM.pdf", sep="") )
pQSFQRpm
dev.off()

#pdf( paste("BootQSFDRpm_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure2A_DR_SM.pdf", sep="") )
pQSFDRpm
dev.off()



rm(ls="results")



# ==============================================
#
#        FULL SAMPLE : Nonlinear QSFs
#
# ==============================================

load("EstimationResultspm_Food_pool_Smplave_logit_ngFS599_ngSS599_ntqsf3_nx5t0_qY0.RData")

# Plot settings
xlim.nl        <- c(min(results$nonlinQR$gridX)-.1, max(results$nonlinQR$gridX)+.1)
ylim.nl        <- c(0,.4)
zlim.nl        <- c(0,.4)
xlab.nl        <- "Total Expenditure"
ylab.nl        <- "Quantile Index"
zlab.nl        <- "Quantile Structural Function"
jet.colors     <- colorRampPalette( c("blue", "lightblue", "yellow", "orange", "red") )
nbcol          <- 100
color          <- jet.colors(nbcol)

nrz            <- nrow(results$nonlinQR$QSF);
ncz            <- ncol(results$nonlinDR$QSF);
zfacetQR       <- results$nonlinQR$QSF[-1, -1] + results$nonlinQR$QSF[-1, -ncz] + results$nonlinQR$QSF[-nrz, -1] + results$nonlinQR$QSF[-nrz, -ncz]     
zfacetDR       <- results$nonlinDR$QSF[-1, -1] + results$nonlinDR$QSF[-1, -ncz] + results$nonlinDR$QSF[-nrz, -1] + results$nonlinDR$QSF[-nrz, -ncz]     
facetcolQR     <- cut(zfacetQR, nbcol)       # Recode facet z-values into color indices;
facetcolDR     <- cut(zfacetDR, nbcol)       # Recode facet z-values into color indices;

tausqsf.nl     <- seq(.15, .85, by = .05);	# Quantiles of interest for the QSF

#pdf( paste(good, "_MainPerspQR_", sample, "_ngy", ntaus2, "_ngx", length(results$nonlin$gridX), ".pdf", sep="") )
pdf( paste("Figure4A_QR.pdf", sep="") )

persp(results$nonlinQR$gridX, tausqsf.nl, results$nonlinQR$QSF, col = color[facetcolQR], xlab = xlab.nl, ylab = ylab.nl, zlab = zlab.nl, zlim = zlim.nl, theta = -40, phi = 20, ticktype = "detailed", expand = .9, border = NA)

dev.off()



#pdf( paste(good, "_MainPerspDR_", sample, "_ngy", ntaus2, "_ngx", length(results$nonlin$gridX), ".pdf", sep="") )
pdf( paste("Figure4A_DR.pdf", sep="") )

persp(results$nonlinDR$gridX, tausqsf.nl, results$nonlinDR$QSF, col = color[facetcolDR], xlab = xlab.nl, ylab = ylab.nl, zlab = zlab.nl, zlim = zlim.nl, theta = -40, phi = 20, ticktype = "detailed", expand = .9, border = NA)

dev.off()

rm(ls="results")



# ==============================================
#
#        FULL SAMPLE : Quad. and cubic CF
#
# ==============================================

ngridX        <- 5
ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf) 

load("EstimationResultspm_Food_pool_Smplave_logit_ngFS599_ngSS599_ntqsf3_nx5t0_qY0.RData")
fplot.lin     <- sfuncsplot(results = results, ngx = nxx, yy = yy)
fplot.quad    <- sfuncsplot_polys(resultsQR = results$QRquad, resultsDR = results$DRquad, ngx = nxx, yy = yy)
fplot.cub     <- sfuncsplot_polys(resultsQR = results$QRcub, resultsDR = results$DRcub, ngx = nxx, yy = yy)

xx            <- seq(results$DR$gridX[1],results$DR$gridX[ngridX],length.out = nxx)
xlim          <- c(min(results$DR$gridX)-.05, max(results$DR$gridX)+.05)



#pdf( paste("QRSens_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),".pdf", sep="") )
pdf( paste("Figure6A_QR_SM.pdf", sep="") )

plot(xlim, ylim.qsf, xlab = "Total Expenditure", ylab = ylab.qsf, type = "n");

lines(xx, fplot.lin$qsfQRfunx[1:nxx],ylim=ylim.qsf,col=2,lwd=2)
lines(xx, fplot.lin$qsfQRfunx[(nxx+1):(2*nxx)],col=2,lwd=2)
lines(xx, fplot.lin$qsfQRfunx[(2*nxx+1):(3*nxx)],col=2,lwd=2)
lines(xx, fplot.quad$qsfQRfunx[1:nxx],col=4,lwd=2)
lines(xx, fplot.quad$qsfQRfunx[(nxx+1):(2*nxx)],col=4,lwd=2)
lines(xx, fplot.quad$qsfQRfunx[(2*nxx+1):(3*nxx)],col=4,lwd=2)
lines(xx, fplot.cub$qsfQRfunx[1:nxx],col=3,lwd=2)
lines(xx, fplot.cub$qsfQRfunx[(nxx+1):(2*nxx)],col=3,lwd=2)
lines(xx, fplot.cub$qsfQRfunx[(2*nxx+1):(3*nxx)],col=3,lwd=2)

dev.off()

#pdf( paste("DRSens_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),".pdf", sep="") )
pdf( paste("Figure6A_DR_SM.pdf", sep="") )

plot(xlim, ylim.qsf, xlab = "Total Expenditure", ylab = ylab.qsf, type = "n");

lines(xx, fplot.lin$qsfDRfunx[1:nxx],ylim=ylim.qsf,col=2,lwd=2)
lines(xx, fplot.lin$qsfDRfunx[(nxx+1):(2*nxx)],col=2,lwd=2)
lines(xx, fplot.lin$qsfDRfunx[(2*nxx+1):(3*nxx)],col=2,lwd=2)
lines(xx, fplot.quad$qsfDRfunx[1:nxx],col=4,lwd=2)
lines(xx, fplot.quad$qsfDRfunx[(nxx+1):(2*nxx)],col=4,lwd=2)
lines(xx, fplot.quad$qsfDRfunx[(2*nxx+1):(3*nxx)],col=4,lwd=2)
lines(xx, fplot.cub$qsfDRfunx[1:nxx],col=3,lwd=2)
lines(xx, fplot.cub$qsfDRfunx[(nxx+1):(2*nxx)],col=3,lwd=2)
lines(xx, fplot.cub$qsfDRfunx[(2*nxx+1):(3*nxx)],col=3,lwd=2)

dev.off()

rm(ls="results")



# ==============================================
#
#        FULL SAMPLE : integration by simulation
#
# ==============================================


integration   <- "simu"                 


# ngridX = 5; ntqsf = 3
# ------------------------------------

ngridX        <- 5
ntausqsf      <- 3;         
tausqsf       <- seq(1/(ntausqsf+1), ntausqsf/(ntausqsf+1), length.out = ntausqsf) 

load("EstimationResultspm_Food_pool_Smplave_logit_ngFS599_ngSS599_ntqsf3_nx5t0_qY0.RData")
fplot         <- sfuncsplot(results = results, ngx = nxx, yy = yy)
load("EstimationResultspm_Food_pool_simu_logit_ngFS599_ngSS599_ntqsf3_nx5t0_qY0.RData")
fplotsimu     <- sfuncsplot(results = results, ngx = nxx, yy = yy)

xx            <- seq(results$DR$gridX[1],results$DR$gridX[ngridX],length.out = nxx)
xlim          <- c(min(results$DR$gridX)-.05, max(results$DR$gridX)+.05)

QSFQR         <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,1,3), each=nxx)), mean = fplot$qsfQRfunx, lwr = fplot$qsfQRfunx.lbw, upr=fplot$qsfQRfunx.ubw )

QSFDR         <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,1,3), each=nxx)), mean = fplot$qsfDRfunx, lwr = fplot$qsfDRfunx.lbw, upr=fplot$qsfDRfunx.ubw )

QSFQRsimu     <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,1,3), each=nxx)), mean = fplotsimu$qsfQRfunx, lwr = fplotsimu$qsfQRfunx.lbw, upr=fplotsimu$qsfQRfunx.ubw )

QSFDRsimu     <- data.frame(Xgrid = rep(xx,ntausqsf), colors = factor(rep(c(2,1,3), each=nxx)), mean = fplotsimu$qsfDRfunx, lwr = fplotsimu$qsfDRfunx.lbw, upr=fplotsimu$qsfDRfunx.ubw )


#pdf( paste("BootQSFQRpmsimu_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure8A_QR_SM.pdf", sep="") )

plot(xlim, ylim.qsf, xlab = "Total Expenditure", ylab = ylab.qsf, type = "n");

lines(xx, fplot$qsfQRfunx[1:nxx],ylim=ylim.qsf,col=2,lwd=2)
lines(xx, fplot$qsfQRfunx[(nxx+1):(2*nxx)],col=2,lwd=2)
lines(xx, fplot$qsfQRfunx[(2*nxx+1):(3*nxx)],col=2,lwd=2)
lines(xx, fplotsimu$qsfQRfunx[1:nxx],col=4,lwd=2)
lines(xx, fplotsimu$qsfQRfunx[(nxx+1):(2*nxx)],col=4,lwd=2)
lines(xx, fplotsimu$qsfQRfunx[(2*nxx+1):(3*nxx)],col=4,lwd=2)

dev.off()


#pdf( paste("BootQSFDRpmsimu_",good,"_",sample,"_",integration,"_",link,"_ngFS",ntaus1,"_ngSS",ntaus2,"_ntqsf",ntausqsf,"_nx",ngridX,"t",length(trim),"_gg.pdf", sep="") )
pdf( paste("Figure8A_DR_SM.pdf", sep="") )

plot(xlim, ylim.qsf, xlab = "Total Expenditure", ylab = ylab.qsf, type = "n");

lines(xx, fplot$qsfDRfunx[1:nxx],ylim=ylim.qsf,col=2,lwd=2)
lines(xx, fplot$qsfDRfunx[(nxx+1):(2*nxx)],col=2,lwd=2)
lines(xx, fplot$qsfDRfunx[(2*nxx+1):(3*nxx)],col=2,lwd=2)
lines(xx, fplotsimu$qsfDRfunx[1:nxx],col=4,lwd=2)
lines(xx, fplotsimu$qsfDRfunx[(nxx+1):(2*nxx)],col=4,lwd=2)
lines(xx, fplotsimu$qsfDRfunx[(2*nxx+1):(3*nxx)],col=4,lwd=2)

dev.off()


#rm(ls="results")


