# This is the file generating Figure 9 in the supplementary material,
# comparing original DGP and adjusted DGP for DR.

# This version : June 2019

#rm(list=ls())

library(foreign);
library(quantreg);

# Set working directory, load relevant functions and data
setwd("")

source("BinLib.R")
source("mainfuncs.R")
source("rearrangement.R")
source("drfunc.R")



#===============================================================================
#                                              
#  		          	CALIBRATION                            
#                                              
#===============================================================================

# Version
#--------------------------------------------------------------------
# Which sample, which good ?

sample        <- "pool"
good          <- "Leisure"
nsim          <- c(1655)      			 # Simulated sample size
correl        <- "calib"
pfac          <- 1

eps           <- 0.01
link          <- "logit"                # For distribution regression                   # number of QR grid points in first stage
trim          <- c(0,1)
ngridY        <- 599
ntaus1        <- 599;                      # number of QR grid points in first stage
ntaus2        <- 599;                      # number of QR grid points in second stage
qY            <- seq(0, 1, length.out = ngridY)
taus1         <- seq(eps, 1-eps, length.out = ntaus1)              
taus2         <- seq(eps, 1-eps, length.out = ntaus2);

#===============================================================================
# Data
#===============================================================================

if(sample == "pool"){ dat  <- read.dta("engelpool.dta") }
attach(dat) 

if(good == "Leisure") { Y <- leisure }
X  	          <- logexp;
Z             <- logwages;



if(good == "Leisure"){ load("CalibrationResults_Leisureng599.Rdata") }

cal.loc <- cal[[1]]
cal.qr  <- cal[[2]]
cal.dr  <- cal[[3]]
sdv <- cal.loc$sdv
sde <- cal.loc$sde

a.loc       <- matrix(cal.loc$avec, 1, 4)
a.qr        <- matrix(cal.qr$avec, 1, 4)
a.dr        <- matrix(cal.dr$avec, 1, 4)
b.loc       <- matrix(cal.loc$bvec, 1, 4)
b.qr        <- matrix(cal.qr$bvec, 1, 4)
b.dr        <- matrix(cal.dr$bvec, 1, 4)

b.dr0       <- b.dr
b.dr[3]     <- b.dr[3]*1.5
b.dr[4]     <- b.dr[4]*2.25


# Plots

gridX <- quantile(X,seq(.1,.9,length.out=5))

xx    <- seq(min(gridX), max(gridX), by = .01)
xxmat <- rbind(1,xx)

pdf( paste("Figure9_SM.pdf", sep="") )

plot(xx, ( -(b.dr[1:2] %*% xxmat)/(b.dr[3:4] %*% xxmat) ) + (1/(b.dr[3:4] %*% xxmat))*qnorm(.5), type = "l", col = 2, ylim=c(-.05,.5), lwd = 2, ylab = "Quantile Structural Function", xlab = "Total Expenditure" )

lines(xx, ( -(b.dr[1:2] %*% xxmat)/(b.dr[3:4] %*% xxmat) ) + (1/(b.dr[3:4] %*% xxmat))*qnorm(.25), col = 2, lwd = 2 )
lines(xx, ( -(b.dr[1:2] %*% xxmat)/(b.dr[3:4] %*% xxmat) ) + (1/(b.dr[3:4] %*% xxmat))*qnorm(.75), col = 2, lwd = 2 )
lines(xx, ( -(b.dr0[1:2] %*% xxmat)/(b.dr0[3:4] %*% xxmat) ) + (1/(b.dr0[3:4] %*% xxmat))*qnorm(.5), col = 1, lwd = 2 )
lines(xx, ( -(b.dr0[1:2] %*% xxmat)/(b.dr0[3:4] %*% xxmat) ) + (1/(b.dr0[3:4] %*% xxmat))*qnorm(.25), col = 1, lwd = 2 )
lines(xx, ( -(b.dr0[1:2] %*% xxmat)/(b.dr0[3:4] %*% xxmat) ) + (1/(b.dr0[3:4] %*% xxmat))*qnorm(.75), col = 1, lwd = 2 )

dev.off()




